module eth_wrapper #(
    parameter DATA_WIDTH = 64,
    parameter KEEP_WIDTH = (DATA_WIDTH/8)
)
(
    // clock and reset
    input                           clk_100,
    input                           reset,


    // apb config interface
    input                           s_apb_mac_psel,
    input                           s_apb_mac_penable,
    input                           s_apb_mac_pwrite,
    input   [2:0]                   s_apb_mac_pprot,
    input   [31:0]                  s_apb_mac_paddr,
    input   [31:0]                  s_apb_mac_pwdata,
    input   [3:0]                   s_apb_mac_pstrb,
    output                          s_apb_mac_pready,
    output  [31:0]                  s_apb_mac_prdata,
    output                          s_apb_mac_pslverr,

    input   [DATA_WIDTH-1:0]        tx_axis_tdata,
    input                           tx_axis_tvalid,
    output                          tx_axis_tready,
    input                           tx_axis_tlast,
    input                           tx_axis_tuser,
    input   [KEEP_WIDTH-1:0]        tx_axis_tkeep,

    output  [DATA_WIDTH-1:0]        rx_axis_tdata,
    output                          rx_axis_tvalid,
    input                           rx_axis_tready,
    output                          rx_axis_tlast,
    output                          rx_axis_tuser,
    output  [KEEP_WIDTH-1:0]        rx_axis_tkeep,

    //MII Interface
    input   wire                    mii_rx_clk,
    input   wire    [3:0]           mii_rxd,
    input   wire                    mii_rx_dv,
    input   wire                    mii_rx_er,
    input   wire                    mii_tx_clk,
    output  wire    [3:0]           mii_txd,
    output  wire                    mii_tx_en,
    output  wire                    mii_tx_er,

    // MDIO Interface
    input                           Mdi,
    output                          Mdo,
    output                          MdoEn,
    output                          Mdc
);


logic   [3:0]   mii_rxd_mux;
logic           mii_rx_dv_mux;
logic           mii_rx_er_mux;

logic           mii_loopback;


logic tx_error_underflow;
logic tx_fifo_overflow;
logic tx_fifo_bad_frame;
logic tx_fifo_good_frame;
logic rx_error_bad_frame;
logic rx_error_bad_fcs;
logic rx_fifo_overflow;
logic rx_fifo_bad_frame;
logic rx_fifo_good_frame;

eth_mac_mii_fifo #(
    .ENABLE_PADDING(1),
    .MIN_FRAME_LENGTH(64),
    .AXIS_DATA_WIDTH(DATA_WIDTH)
) u_eth_mac(
    .rst                (reset),
    .logic_clk          (clk_100),
    .logic_rst          (reset),

    .tx_axis_tdata      (tx_axis_tdata),
    .tx_axis_tkeep      (tx_axis_tkeep),
    .tx_axis_tvalid     (tx_axis_tvalid),
    .tx_axis_tready     (tx_axis_tready),
    .tx_axis_tlast      (tx_axis_tlast),
    .tx_axis_tuser      ('0),

    .rx_axis_tdata      (rx_axis_tdata),
    .rx_axis_tkeep      (rx_axis_tkeep),
    .rx_axis_tvalid     (rx_axis_tvalid),
    .rx_axis_tready     (rx_axis_tready),
    .rx_axis_tlast      (rx_axis_tlast),
    .rx_axis_tuser      (rx_axis_tuser),

    .mii_rx_clk         (mii_rx_clk),
    .mii_rxd            (mii_rxd_mux),
    .mii_rx_dv          (mii_rx_dv_mux),
    .mii_rx_er          (mii_rx_er_mux),
    .mii_tx_clk         (mii_tx_clk),
    .mii_txd            (mii_txd),
    .mii_tx_en          (mii_tx_en),
    .mii_tx_er          (mii_tx_er),

    .tx_error_underflow (tx_error_underflow),
    .tx_fifo_overflow   (tx_fifo_overflow),
    .tx_fifo_bad_frame  (tx_fifo_bad_frame),
    .tx_fifo_good_frame (tx_fifo_good_frame),
    .rx_error_bad_frame (rx_error_bad_frame),
    .rx_error_bad_fcs   (rx_error_bad_fcs),
    .rx_fifo_overflow   (rx_fifo_overflow),
    .rx_fifo_bad_frame  (rx_fifo_bad_frame),
    .rx_fifo_good_frame (rx_fifo_good_frame),

    .cfg_ifg            (24),
    .cfg_tx_enable      ('1),   // this should be configurable w/ regfile
    .cfg_rx_enable      ('1)
);

mac_regs_pkg::mac_regs__in_t hwif_in;
mac_regs_pkg::mac_regs__out_t hwif_out;

mac_regs u_mac_regs(
    .clk(clk_100),
    .rst(reset),
    .s_apb_psel(s_apb_mac_psel),
    .s_apb_penable(s_apb_mac_penable),
    .s_apb_pwrite(s_apb_mac_pwrite),
    .s_apb_pprot(s_apb_mac_pprot),
    .s_apb_paddr(s_apb_mac_paddr),
    .s_apb_pwdata(s_apb_mac_pwdata),
    .s_apb_pstrb(s_apb_mac_pstrb),
    .s_apb_pready(s_apb_mac_pready),
    .s_apb_prdata(s_apb_mac_prdata),
    .s_apb_pslverr(s_apb_mac_pslverr),

    .hwif_in(hwif_in),
    .hwif_out(hwif_out)
);

assign mii_loopback = hwif_out.mii_loopback.mii_loopback_en.value;

assign hwif_in.mac_status.tx_error_underflow.hwset = tx_error_underflow;
assign hwif_in.mac_status.tx_fifo_overflow.hwset = tx_fifo_overflow;
assign hwif_in.mac_status.tx_fifo_bad_frame.hwset = tx_fifo_bad_frame;
assign hwif_in.mac_status.tx_fifo_good_frame.hwset = tx_fifo_good_frame;
assign hwif_in.mac_status.rx_error_bad_frame.hwset = rx_error_bad_frame;
assign hwif_in.mac_status.rx_error_bad_fcs.hwset = rx_error_bad_fcs;
assign hwif_in.mac_status.rx_fifo_overflow.hwset = rx_fifo_overflow;
assign hwif_in.mac_status.rx_fifo_bad_frame.hwset = rx_fifo_bad_frame;
assign hwif_in.mac_status.rx_fifo_good_frame.hwset = rx_fifo_good_frame;

always_comb begin
    if (mii_loopback) begin
        mii_rxd_mux = mii_txd;
        mii_rx_dv_mux = mii_tx_en;
        mii_rx_er_mux = mii_tx_er;
    end else begin
        mii_rxd_mux = mii_rxd;
        mii_rx_dv_mux = mii_rx_dv;
        mii_rx_er_mux = mii_rx_er;
    end
end


endmodule