module control_client #(
    parameter DATA_WIDTH = 8,
    parameter KEEP_WIDTH = (DATA_WIDTH/8),
    parameter LOCAL_MAC = 48'h020000aabbcc, // should be a register setting
    parameter LOCAL_IP  = 32'hac000002      // should be a register setting
)
(
    input  wire        clk,
    input  wire        rst,

    /*
     * UDP frame input
     */
    input  wire                     s_udp_hdr_valid,
    output wire                     s_udp_hdr_ready,
    input  wire [47:0]              s_eth_dest_mac,
    input  wire [47:0]              s_eth_src_mac,
    input  wire [15:0]              s_eth_type,
    input  wire [3:0]               s_ip_version,
    input  wire [3:0]               s_ip_ihl,
    input  wire [5:0]               s_ip_dscp,
    input  wire [1:0]               s_ip_ecn,
    input  wire [15:0]              s_ip_identification,
    input  wire [2:0]               s_ip_flags,
    input  wire [12:0]              s_ip_fragment_offset,
    input  wire [7:0]               s_ip_ttl,
    input  wire [7:0]               s_ip_protocol,
    input  wire [15:0]              s_ip_header_checksum,
    input  wire [31:0]              s_ip_source_ip,
    input  wire [31:0]              s_ip_dest_ip,
    input  wire [15:0]              s_udp_source_port,
    input  wire [15:0]              s_udp_dest_port,
    input  wire [15:0]              s_udp_length,
    input  wire [15:0]              s_udp_checksum,
    input  wire [DATA_WIDTH-1:0]    s_udp_payload_axis_tdata,
    input  wire [KEEP_WIDTH-1:0]    s_udp_payload_axis_tkeep,
    input  wire                     s_udp_payload_axis_tvalid,
    output wire                     s_udp_payload_axis_tready,
    input  wire                     s_udp_payload_axis_tlast,
    input  wire                     s_udp_payload_axis_tuser,

    /*
     * UDP frame output
     */
    output wire                     m_udp_hdr_valid,
    input  wire                     m_udp_hdr_ready,
    output wire [47:0]              m_eth_dest_mac,
    output wire [47:0]              m_eth_src_mac,
    output wire [15:0]              m_eth_type,
    output wire [3:0]               m_ip_version,
    output wire [3:0]               m_ip_ihl,
    output wire [5:0]               m_ip_dscp,
    output wire [1:0]               m_ip_ecn,
    output wire [15:0]              m_ip_identification,
    output wire [2:0]               m_ip_flags,
    output wire [12:0]              m_ip_fragment_offset,
    output wire [7:0]               m_ip_ttl,
    output wire [7:0]               m_ip_protocol,
    output wire [15:0]              m_ip_header_checksum,
    output wire [31:0]              m_ip_source_ip,
    output wire [31:0]              m_ip_dest_ip,
    output wire [15:0]              m_udp_source_port,
    output wire [15:0]              m_udp_dest_port,
    output wire [15:0]              m_udp_length,
    output wire [15:0]              m_udp_checksum,
    output wire [DATA_WIDTH-1:0]    m_udp_payload_axis_tdata,
    output wire [KEEP_WIDTH-1:0]    m_udp_payload_axis_tkeep,
    output wire                     m_udp_payload_axis_tvalid,
    input  wire                     m_udp_payload_axis_tready,
    output wire                     m_udp_payload_axis_tlast,
    output wire                     m_udp_payload_axis_tuser,

    // APB Master
    output wire [31:0]              m_apb_paddr,
    output wire                     m_apb_psel,
    output wire                     m_apb_penable,
    output wire                     m_apb_pwrite,
    output wire [31:0]              m_apb_pwdata,
    output wire [3:0]               m_apb_pstrb,
    input  wire                     m_apb_pready,
    input  wire [31:0]              m_apb_prdata,
    input  wire                     m_apb_pslverr

);

    localparam HDR_FIFO_WIDTH = (16+16+16+32+48+16+16+16);
    localparam DATA_FIFO_WIDTH = (DATA_WIDTH+KEEP_WIDTH+1+1);

    enum logic {HEADER, DATA} tx_state, tx_state_next;

    logic [15:0] ip_new_checksum;
    logic [15:0] udp_new_checksum;


    logic udp_hdr_ready;
    assign s_udp_hdr_ready = udp_hdr_ready;

    logic udp_hdr_valid;

    logic [7:0]  operation_reg, operation_next;
    logic [31:0] address_reg, address_next;
    logic [31:0] wdata_reg, wdata_next;
    logic [31:0] rdata_reg, rdata_next;


    logic [31:0] apb_paddr;
    logic        apb_psel;
    logic        apb_penable;
    logic        apb_pwrite;
    logic [31:0] apb_pwdata;
    logic [3:0]  apb_pstrb;

    logic penable_next;

    assign m_apb_paddr = apb_paddr;
    assign m_apb_psel = apb_psel;
    assign m_apb_penable = apb_penable;
    assign m_apb_pwrite = apb_pwrite;
    assign m_apb_pwdata = apb_pwdata;
    assign m_apb_pstrb = apb_pstrb;

    logic checksum_clear;
    logic [31:0] checksum_data;
    logic [15:0] checksum_out;


    // Header fifo stores:
    // IP Identification    16
    // IP Dest Address      32 (originally the source address)
    // ETH Dest Address     48 (originally the source address)
    // UDP Dest Port        16 (originally the source port)


    logic [15:0] ip_ident_reg, ip_ident_next;
    logic [31:0] ip_dest_reg, ip_dest_next;
    logic [47:0] eth_dest_reg, eth_dest_next;
    logic [15:0] udp_dest_reg, udp_dest_next;
    logic [15:0] orig_dest_reg, orig_dest_next;

    enum logic [1:0] {MSG_RX, APB_ACCESS, CHECKSUM_CALC, MSG_TX} state, state_next;

    logic [3:0] counter, counter_next;

    logic s_tready;

    assign s_udp_payload_axis_tready = s_tready;

    logic udp_payload_valid;
    logic [7:0] udp_payload_axis_tdata; // TODO make this data width dependent
    logic udp_payload_axis_tlast;

    assign m_udp_hdr_valid = udp_hdr_valid;
    assign m_udp_payload_axis_tvalid = udp_payload_valid;
    assign m_udp_payload_axis_tdata = udp_payload_axis_tdata;
    assign m_udp_payload_axis_tlast = udp_payload_axis_tlast;
    assign m_udp_payload_axis_tuser = '0;

    assign m_eth_dest_mac = eth_dest_reg;
    assign m_eth_src_mac = LOCAL_MAC;
    assign m_eth_type = 16'h0800;
    assign m_udp_source_port = 16'd8000;
    assign m_udp_dest_port = udp_dest_reg;
    assign m_udp_checksum = checksum_out;
    assign m_ip_dscp = 6'h0;
    assign m_ip_ecn = 2'h0;
    assign m_ip_identification = ip_ident_reg;
    assign m_ip_flags = 3'b010;
    assign m_ip_fragment_offset = 13'h0;
    assign m_ip_ttl = 8'h40;
    assign m_ip_protocol = 8'h11;
    assign m_ip_version = 4'h4;
    assign m_ip_ihl = 4'h5;
    assign m_ip_source_ip = LOCAL_IP;
    assign m_ip_dest_ip = ip_dest_reg;
    assign m_udp_length = 16'h14;


generate
    if (DATA_WIDTH == 64) begin
        $error("64 bit control client not supported yet");
    end else if (DATA_WIDTH == 8) begin

        always_ff @(posedge clk) begin
            if (rst) begin
                counter <= '0;
                state <= MSG_RX;
                apb_penable <= '0;
                rdata_reg <= '0;
            end else begin
                counter <= counter_next;
                state <= state_next;
                apb_penable <= penable_next;

                operation_reg <= operation_next;
                address_reg <= address_next;
                wdata_reg <= wdata_next;
                rdata_reg <= rdata_next;

                eth_dest_reg <= eth_dest_next;
                ip_ident_reg <= ip_ident_next;
                ip_dest_reg <= ip_dest_next;
                udp_dest_reg <= udp_dest_next;
                orig_dest_reg <= orig_dest_next;
            end
        end

        always_comb begin
            operation_next = operation_reg;
            address_next = address_reg;
            wdata_next = wdata_reg;
            state_next = state;
            counter_next = counter;
            penable_next = '0;

            s_tready = '0;

            rdata_next = rdata_reg;

            apb_paddr   = '0;
            apb_psel    = '0;
            apb_pwrite  = '0;
            apb_pwdata  = '0;
            apb_pstrb   = '0;

            checksum_clear = '0;
            checksum_data = '0;

            udp_hdr_ready = '0;
            udp_hdr_valid = '0;

            udp_payload_valid = '0;
            udp_payload_axis_tdata = '0;
            udp_payload_axis_tlast = '0;

            eth_dest_next = eth_dest_reg;
            ip_ident_next = ip_ident_reg;
            ip_dest_next = ip_dest_reg;
            udp_dest_next = udp_dest_reg;
            orig_dest_next = orig_dest_reg;

            case (state)
                MSG_RX:
                    begin
                        if (s_udp_hdr_valid) begin
                            udp_hdr_ready = '1;

                            eth_dest_next = s_eth_src_mac;
                            ip_ident_next = s_ip_identification;
                            ip_dest_next = s_ip_source_ip;
                            udp_dest_next = s_udp_source_port;
                            orig_dest_next = s_udp_dest_port;
                        end


                        if (s_udp_payload_axis_tvalid) begin
                            s_tready = '1;
                            counter_next = counter + 1;

                            case (counter)
                                0:  operation_next      = s_udp_payload_axis_tdata;
                                //  1, 2, 3
                                4:  address_next[31:24] = s_udp_payload_axis_tdata;
                                5:  address_next[23:16] = s_udp_payload_axis_tdata;
                                6:  address_next[15: 8] = s_udp_payload_axis_tdata;
                                7:  address_next[ 7: 0] = s_udp_payload_axis_tdata;
                                8:  wdata_next[31:24]   = s_udp_payload_axis_tdata;
                                9:  wdata_next[23:16]   = s_udp_payload_axis_tdata;
                                10: wdata_next[15: 8]   = s_udp_payload_axis_tdata;
                                11: begin
                                        wdata_next[ 7: 0] = s_udp_payload_axis_tdata;
                                        // only continue if the port is 8000
                                        if (orig_dest_reg == 16'd8000) begin
                                            state_next = APB_ACCESS;
                                        end
                                    end
                            endcase

                            // If we receive a valid packet we should be going to next state anyway
                            if (s_udp_payload_axis_tlast) begin
                                counter_next = '0;
                            end
                        end
                    end

                APB_ACCESS:
                    begin
                        counter_next = '0;

                        penable_next = '1;

                        apb_paddr = address_reg;
                        apb_psel = '1;
                        apb_pwrite = (operation_reg == 8'b1) ? 1 : 0;
                        apb_pwdata = wdata_reg;
                        apb_pstrb = '1;

                        if (apb_penable && m_apb_pready) begin
                            penable_next = '0;
                            checksum_clear = '1;
                            rdata_next = m_apb_prdata;
                            state_next = CHECKSUM_CALC;
                        end
                    end

                CHECKSUM_CALC: begin
                    counter_next = counter + 1;
                    case (counter)
                        0:  checksum_data = m_ip_source_ip;
                        1:  checksum_data = m_ip_dest_ip;
                        2:  checksum_data = {8'h0, 8'h11, 16'd20};
                        3:  checksum_data = m_udp_source_port;
                        4:  checksum_data = m_udp_dest_port;
                        5:  checksum_data = 32'd20;
                        6:  checksum_data = {8'h2, 24'b0};
                        7:  checksum_data = address_reg;
                        8:  begin
                                checksum_data = rdata_reg;
                                state_next = MSG_TX;
                                counter_next = '0;
                            end
                    endcase
                end

                MSG_TX:
                    begin
                        if (counter == 0) begin
                            udp_hdr_valid = '1;
                            if (m_udp_hdr_ready) begin
                                counter_next = counter+1;
                            end
                        end else begin
                            udp_payload_valid = '1;

                            if (m_udp_payload_axis_tready) begin
                                counter_next = counter + 1;
                            end

                            case (counter)
                                1:  udp_payload_axis_tdata = 8'h2;
                                2:  udp_payload_axis_tdata = 8'h0;
                                3:  udp_payload_axis_tdata = 8'h0;
                                4:  udp_payload_axis_tdata = 8'h0;
                                5:  udp_payload_axis_tdata = address_reg[31:24];
                                6:  udp_payload_axis_tdata = address_reg[23:16];
                                7:  udp_payload_axis_tdata = address_reg[15: 8];
                                8:  udp_payload_axis_tdata = address_reg[ 7: 0];
                                9:  udp_payload_axis_tdata = rdata_reg[31:24];
                                10: udp_payload_axis_tdata = rdata_reg[23:16];
                                11: udp_payload_axis_tdata = rdata_reg[15: 8];
                                12: begin
                                        udp_payload_axis_tdata = rdata_reg[ 7: 0];
                                        udp_payload_axis_tlast = '1;
                                    end
                                13: begin
                                        state_next = MSG_RX;
                                        counter_next = '0;
                                        udp_payload_valid = '0;
                                    end
                            endcase
                        end
                    end
            endcase
        end

    end else begin
        $error("Only 8 and 64 bit data widths are supported");
    end
endgenerate

    checksum_calc u_control_client_checksum_calc (
        .i_rst      (rst),
        .i_clk      (clk),

        .i_clear    (checksum_clear),
        .i_data     (checksum_data),
        .o_checksum (checksum_out)
    );

endmodule