module control_wrapper#(
    parameter DATA_WIDTH = 64,
    parameter KEEP_ENABLE = (DATA_WIDTH>8),
    parameter KEEP_WIDTH = (DATA_WIDTH/8),
    parameter USER_WIDTH = 1
)
(
    input  wire                         clk,
    input  wire                         rst,

    /*
     * IP frame input
     */
    input  wire                         s_ip_hdr_valid,
    output wire                         s_ip_hdr_ready,
    input  wire [47:0]                  s_eth_dest_mac,
    input  wire [47:0]                  s_eth_src_mac,
    input  wire [15:0]                  s_eth_type,
    input  wire [3:0]                   s_ip_version,
    input  wire [3:0]                   s_ip_ihl,
    input  wire [5:0]                   s_ip_dscp,
    input  wire [1:0]                   s_ip_ecn,
    input  wire [15:0]                  s_ip_length,
    input  wire [15:0]                  s_ip_identification,
    input  wire [2:0]                   s_ip_flags,
    input  wire [12:0]                  s_ip_fragment_offset,
    input  wire [7:0]                   s_ip_ttl,
    input  wire [7:0]                   s_ip_protocol,
    input  wire [15:0]                  s_ip_header_checksum,
    input  wire [31:0]                  s_ip_source_ip,
    input  wire [31:0]                  s_ip_dest_ip,
    input  wire [DATA_WIDTH-1:0]        s_ip_payload_axis_tdata,
    input  wire [KEEP_WIDTH-1:0]        s_ip_payload_axis_tkeep,
    input  wire                         s_ip_payload_axis_tvalid,
    output wire                         s_ip_payload_axis_tready,
    input  wire                         s_ip_payload_axis_tlast,
    input  wire [USER_WIDTH-1:0]        s_ip_payload_axis_tuser,

    /*
     * IP frame output
     */
    output wire                         m_ip_hdr_valid,
    input  wire                         m_ip_hdr_ready,
    output wire [47:0]                  m_eth_dest_mac,
    output wire [47:0]                  m_eth_src_mac,
    output wire [15:0]                  m_eth_type,
    output wire [3:0]                   m_ip_version,
    output wire [3:0]                   m_ip_ihl,
    output wire [5:0]                   m_ip_dscp,
    output wire [1:0]                   m_ip_ecn,
    output wire [15:0]                  m_ip_length,
    output wire [15:0]                  m_ip_identification,
    output wire [2:0]                   m_ip_flags,
    output wire [12:0]                  m_ip_fragment_offset,
    output wire [7:0]                   m_ip_ttl,
    output wire [7:0]                   m_ip_protocol,
    output wire [15:0]                  m_ip_header_checksum,
    output wire [31:0]                  m_ip_source_ip,
    output wire [31:0]                  m_ip_dest_ip,
    output wire [DATA_WIDTH-1:0]        m_ip_payload_axis_tdata,
    output wire [KEEP_WIDTH-1:0]        m_ip_payload_axis_tkeep,
    output wire                         m_ip_payload_axis_tvalid,
    input  wire                         m_ip_payload_axis_tready,
    output wire                         m_ip_payload_axis_tlast,
    output wire                         m_ip_payload_axis_tuser,

    // APB Master
    output wire [31:0]                  m_apb_paddr,
    output wire                         m_apb_psel,
    output wire                         m_apb_penable,
    output wire                         m_apb_pwrite,
    output wire [31:0]                  m_apb_pwdata,
    output wire [3:0]                   m_apb_pstrb,
    input  wire                         m_apb_pready,
    input  wire [31:0]                  m_apb_prdata,
    input  wire                         m_apb_pslverr
);


    logic                       ctrl_rx_udp_hdr_valid;
    logic                       ctrl_rx_udp_hdr_ready;
    logic   [47:0]              ctrl_rx_eth_dest_mac;
    logic   [47:0]              ctrl_rx_eth_src_mac;
    logic   [15:0]              ctrl_rx_eth_type;
    logic   [3:0]               ctrl_rx_ip_version;
    logic   [3:0]               ctrl_rx_ip_ihl;
    logic   [5:0]               ctrl_rx_ip_dscp;
    logic   [1:0]               ctrl_rx_ip_ecn;
    logic   [15:0]              ctrl_rx_ip_length;
    logic   [15:0]              ctrl_rx_ip_identification;
    logic   [2:0]               ctrl_rx_ip_flags;
    logic   [12:0]              ctrl_rx_ip_fragment_offset;
    logic   [7:0]               ctrl_rx_ip_ttl;
    logic   [7:0]               ctrl_rx_ip_protocol;
    logic   [15:0]              ctrl_rx_ip_header_checksum;
    logic   [31:0]              ctrl_rx_ip_source_ip;
    logic   [31:0]              ctrl_rx_ip_dest_ip;
    logic   [15:0]              ctrl_rx_udp_source_port;
    logic   [15:0]              ctrl_rx_udp_dest_port;
    logic   [15:0]              ctrl_rx_udp_length;
    logic   [15:0]              ctrl_rx_udp_checksum;
    logic   [DATA_WIDTH-1:0]    ctrl_rx_udp_payload_axis_tdata;
    logic   [KEEP_WIDTH-1:0]    ctrl_rx_udp_payload_axis_tkeep;
    logic                       ctrl_rx_udp_payload_axis_tvalid;
    logic                       ctrl_rx_udp_payload_axis_tready;
    logic                       ctrl_rx_udp_payload_axis_tlast;
    logic                       ctrl_rx_udp_payload_axis_tuser;

    logic                       ctrl_rx_busy;
    logic                       ctrl_rx_error_header_early_termination;
    logic                       ctrl_rx_error_payload_early_termination;

    logic                       ctrl_tx_udp_hdr_valid;
    logic                       ctrl_tx_udp_hdr_ready;
    logic   [47:0]              ctrl_tx_eth_dest_mac;
    logic   [47:0]              ctrl_tx_eth_src_mac;
    logic   [15:0]              ctrl_tx_eth_type;
    logic   [3:0]               ctrl_tx_ip_version;
    logic   [3:0]               ctrl_tx_ip_ihl;
    logic   [5:0]               ctrl_tx_ip_dscp;
    logic   [1:0]               ctrl_tx_ip_ecn;
    logic   [15:0]              ctrl_tx_ip_length;
    logic   [15:0]              ctrl_tx_ip_identification;
    logic   [2:0]               ctrl_tx_ip_flags;
    logic   [12:0]              ctrl_tx_ip_fragment_offset;
    logic   [7:0]               ctrl_tx_ip_ttl;
    logic   [7:0]               ctrl_tx_ip_protocol;
    logic   [15:0]              ctrl_tx_ip_header_checksum;
    logic   [31:0]              ctrl_tx_ip_source_ip;
    logic   [31:0]              ctrl_tx_ip_dest_ip;
    logic   [15:0]              ctrl_tx_udp_source_port;
    logic   [15:0]              ctrl_tx_udp_dest_port;
    logic   [15:0]              ctrl_tx_udp_length;
    logic   [15:0]              ctrl_tx_udp_checksum;
    logic   [DATA_WIDTH-1:0]    ctrl_tx_udp_payload_axis_tdata;
    logic   [KEEP_WIDTH-1:0]    ctrl_tx_udp_payload_axis_tkeep;
    logic                       ctrl_tx_udp_payload_axis_tvalid;
    logic                       ctrl_tx_udp_payload_axis_tready;
    logic                       ctrl_tx_udp_payload_axis_tlast;
    logic                       ctrl_tx_udp_payload_axis_tuser;

    logic                       ctrl_tx_busy;
    logic                       ctrl_tx_error_payload_early_termination;

generate
    if (DATA_WIDTH == 64) begin
        udp_ip_rx_64 u_control_wrapper_udp_ip_rx_64 (
            .clk                                (clk),
            .rst                                (rst),

            .s_ip_hdr_valid                     (s_ip_hdr_valid),
            .s_ip_hdr_ready                     (s_ip_hdr_ready),
            .s_eth_dest_mac                     (s_eth_dest_mac),
            .s_eth_src_mac                      (s_eth_src_mac),
            .s_eth_type                         (s_eth_type),
            .s_ip_version                       (s_ip_version),
            .s_ip_ihl                           (s_ip_ihl),
            .s_ip_dscp                          (s_ip_dscp),
            .s_ip_ecn                           (s_ip_ecn),
            .s_ip_length                        (s_ip_length),
            .s_ip_identification                (s_ip_identification),
            .s_ip_flags                         (s_ip_flags),
            .s_ip_fragment_offset               (s_ip_fragment_offset),
            .s_ip_ttl                           (s_ip_ttl),
            .s_ip_protocol                      (s_ip_protocol),
            .s_ip_header_checksum               (s_ip_header_checksum),
            .s_ip_source_ip                     (s_ip_source_ip),
            .s_ip_dest_ip                       (s_ip_dest_ip),
            .s_ip_payload_axis_tdata            (s_ip_payload_axis_tdata),
            .s_ip_payload_axis_tkeep            (s_ip_payload_axis_tkeep),
            .s_ip_payload_axis_tvalid           (s_ip_payload_axis_tvalid),
            .s_ip_payload_axis_tready           (s_ip_payload_axis_tready),
            .s_ip_payload_axis_tlast            (s_ip_payload_axis_tlast),
            .s_ip_payload_axis_tuser            (s_ip_payload_axis_tuser),

            .m_udp_hdr_valid                    (ctrl_rx_udp_hdr_valid),
            .m_udp_hdr_ready                    (ctrl_rx_udp_hdr_ready),
            .m_eth_dest_mac                     (ctrl_rx_eth_dest_mac),
            .m_eth_src_mac                      (ctrl_rx_eth_src_mac),
            .m_eth_type                         (ctrl_rx_eth_type),
            .m_ip_version                       (ctrl_rx_ip_version),
            .m_ip_ihl                           (ctrl_rx_ip_ihl),
            .m_ip_dscp                          (ctrl_rx_ip_dscp),
            .m_ip_ecn                           (ctrl_rx_ip_ecn),
            .m_ip_length                        (ctrl_rx_ip_length),
            .m_ip_identification                (ctrl_rx_ip_identification),
            .m_ip_flags                         (ctrl_rx_ip_flags),
            .m_ip_fragment_offset               (ctrl_rx_ip_fragment_offset),
            .m_ip_ttl                           (ctrl_rx_ip_ttl),
            .m_ip_protocol                      (ctrl_rx_ip_protocol),
            .m_ip_header_checksum               (ctrl_rx_ip_header_checksum),
            .m_ip_source_ip                     (ctrl_rx_ip_source_ip),
            .m_ip_dest_ip                       (ctrl_rx_ip_dest_ip),
            .m_udp_source_port                  (ctrl_rx_udp_source_port),
            .m_udp_dest_port                    (ctrl_rx_udp_dest_port),
            .m_udp_length                       (ctrl_rx_udp_length),
            .m_udp_checksum                     (ctrl_rx_udp_checksum),
            .m_udp_payload_axis_tdata           (ctrl_rx_udp_payload_axis_tdata),
            .m_udp_payload_axis_tkeep           (ctrl_rx_udp_payload_axis_tkeep),
            .m_udp_payload_axis_tvalid          (ctrl_rx_udp_payload_axis_tvalid),
            .m_udp_payload_axis_tready          (ctrl_rx_udp_payload_axis_tready),
            .m_udp_payload_axis_tlast           (ctrl_rx_udp_payload_axis_tlast),
            .m_udp_payload_axis_tuser           (ctrl_rx_udp_payload_axis_tuser),

            .busy                               (ctrl_rx_busy),
            .error_header_early_termination     (ctrl_rx_error_header_early_termination),
            .error_payload_early_termination    (ctrl_rx_error_payload_early_termination)
        );
    end else if (DATA_WIDTH == 8) begin
        udp_ip_rx u_control_wrapper_udp_ip_rx (
            .clk                                (clk),
            .rst                                (rst),

            .s_ip_hdr_valid                     (s_ip_hdr_valid),
            .s_ip_hdr_ready                     (s_ip_hdr_ready),
            .s_eth_dest_mac                     (s_eth_dest_mac),
            .s_eth_src_mac                      (s_eth_src_mac),
            .s_eth_type                         (s_eth_type),
            .s_ip_version                       (s_ip_version),
            .s_ip_ihl                           (s_ip_ihl),
            .s_ip_dscp                          (s_ip_dscp),
            .s_ip_ecn                           (s_ip_ecn),
            .s_ip_length                        (s_ip_length),
            .s_ip_identification                (s_ip_identification),
            .s_ip_flags                         (s_ip_flags),
            .s_ip_fragment_offset               (s_ip_fragment_offset),
            .s_ip_ttl                           (s_ip_ttl),
            .s_ip_protocol                      (s_ip_protocol),
            .s_ip_header_checksum               (s_ip_header_checksum),
            .s_ip_source_ip                     (s_ip_source_ip),
            .s_ip_dest_ip                       (s_ip_dest_ip),
            .s_ip_payload_axis_tdata            (s_ip_payload_axis_tdata),
            .s_ip_payload_axis_tvalid           (s_ip_payload_axis_tvalid),
            .s_ip_payload_axis_tready           (s_ip_payload_axis_tready),
            .s_ip_payload_axis_tlast            (s_ip_payload_axis_tlast),
            .s_ip_payload_axis_tuser            (s_ip_payload_axis_tuser),

            .m_udp_hdr_valid                    (ctrl_rx_udp_hdr_valid),
            .m_udp_hdr_ready                    (ctrl_rx_udp_hdr_ready),
            .m_eth_dest_mac                     (ctrl_rx_eth_dest_mac),
            .m_eth_src_mac                      (ctrl_rx_eth_src_mac),
            .m_eth_type                         (ctrl_rx_eth_type),
            .m_ip_version                       (ctrl_rx_ip_version),
            .m_ip_ihl                           (ctrl_rx_ip_ihl),
            .m_ip_dscp                          (ctrl_rx_ip_dscp),
            .m_ip_ecn                           (ctrl_rx_ip_ecn),
            .m_ip_length                        (ctrl_rx_ip_length),
            .m_ip_identification                (ctrl_rx_ip_identification),
            .m_ip_flags                         (ctrl_rx_ip_flags),
            .m_ip_fragment_offset               (ctrl_rx_ip_fragment_offset),
            .m_ip_ttl                           (ctrl_rx_ip_ttl),
            .m_ip_protocol                      (ctrl_rx_ip_protocol),
            .m_ip_header_checksum               (ctrl_rx_ip_header_checksum),
            .m_ip_source_ip                     (ctrl_rx_ip_source_ip),
            .m_ip_dest_ip                       (ctrl_rx_ip_dest_ip),
            .m_udp_source_port                  (ctrl_rx_udp_source_port),
            .m_udp_dest_port                    (ctrl_rx_udp_dest_port),
            .m_udp_length                       (ctrl_rx_udp_length),
            .m_udp_checksum                     (ctrl_rx_udp_checksum),
            .m_udp_payload_axis_tdata           (ctrl_rx_udp_payload_axis_tdata),
            .m_udp_payload_axis_tvalid          (ctrl_rx_udp_payload_axis_tvalid),
            .m_udp_payload_axis_tready          (ctrl_rx_udp_payload_axis_tready),
            .m_udp_payload_axis_tlast           (ctrl_rx_udp_payload_axis_tlast),
            .m_udp_payload_axis_tuser           (ctrl_rx_udp_payload_axis_tuser),

            .busy                               (ctrl_rx_busy),
            .error_header_early_termination     (ctrl_rx_error_header_early_termination),
            .error_payload_early_termination    (ctrl_rx_error_payload_early_termination)
        );

        assign ctrl_rx_udp_payload_axis_tkeep = ctrl_rx_udp_payload_axis_tvalid;
    end else begin
        $error("Only 8 and 64 bit data widths are supported");
    end
endgenerate

    control_client #(
        .DATA_WIDTH(8)
    ) u_control_client (
        .clk                                (clk),
        .rst                                (rst),

        .s_udp_hdr_valid                    (ctrl_rx_udp_hdr_valid),
        .s_udp_hdr_ready                    (ctrl_rx_udp_hdr_ready),
        .s_eth_dest_mac                     (ctrl_rx_eth_dest_mac),
        .s_eth_src_mac                      (ctrl_rx_eth_src_mac),
        .s_eth_type                         (ctrl_rx_eth_type),
        .s_ip_version                       (ctrl_rx_ip_version),
        .s_ip_ihl                           (ctrl_rx_ip_ihl),
        .s_ip_dscp                          (ctrl_rx_ip_dscp),
        .s_ip_ecn                           (ctrl_rx_ip_ecn),
        .s_ip_identification                (ctrl_rx_ip_identification),
        .s_ip_flags                         (ctrl_rx_ip_flags),
        .s_ip_fragment_offset               (ctrl_rx_ip_fragment_offset),
        .s_ip_ttl                           (ctrl_rx_ip_ttl),
        .s_ip_protocol                      (ctrl_rx_ip_protocol),
        .s_ip_header_checksum               (ctrl_rx_ip_header_checksum),
        .s_ip_source_ip                     (ctrl_rx_ip_source_ip),
        .s_ip_dest_ip                       (ctrl_rx_ip_dest_ip),
        .s_udp_source_port                  (ctrl_rx_udp_source_port),
        .s_udp_dest_port                    (ctrl_rx_udp_dest_port),
        .s_udp_length                       (ctrl_rx_udp_length),
        .s_udp_checksum                     (ctrl_rx_udp_checksum),
        .s_udp_payload_axis_tdata           (ctrl_rx_udp_payload_axis_tdata),
        .s_udp_payload_axis_tkeep           (ctrl_rx_udp_payload_axis_tkeep),
        .s_udp_payload_axis_tvalid          (ctrl_rx_udp_payload_axis_tvalid),
        .s_udp_payload_axis_tready          (ctrl_rx_udp_payload_axis_tready),
        .s_udp_payload_axis_tlast           (ctrl_rx_udp_payload_axis_tlast),
        .s_udp_payload_axis_tuser           (ctrl_rx_udp_payload_axis_tuser),

        .m_udp_hdr_valid                    (ctrl_tx_udp_hdr_valid),
        .m_udp_hdr_ready                    (ctrl_tx_udp_hdr_ready),
        .m_eth_dest_mac                     (ctrl_tx_eth_dest_mac),
        .m_eth_src_mac                      (ctrl_tx_eth_src_mac),
        .m_eth_type                         (ctrl_tx_eth_type),
        .m_ip_version                       (ctrl_tx_ip_version),
        .m_ip_ihl                           (ctrl_tx_ip_ihl),
        .m_ip_dscp                          (ctrl_tx_ip_dscp),
        .m_ip_ecn                           (ctrl_tx_ip_ecn),
        .m_ip_identification                (ctrl_tx_ip_identification),
        .m_ip_flags                         (ctrl_tx_ip_flags),
        .m_ip_fragment_offset               (ctrl_tx_ip_fragment_offset),
        .m_ip_ttl                           (ctrl_tx_ip_ttl),
        .m_ip_protocol                      (ctrl_tx_ip_protocol),
        .m_ip_header_checksum               (ctrl_tx_ip_header_checksum),
        .m_ip_source_ip                     (ctrl_tx_ip_source_ip),
        .m_ip_dest_ip                       (ctrl_tx_ip_dest_ip),
        .m_udp_source_port                  (ctrl_tx_udp_source_port),
        .m_udp_dest_port                    (ctrl_tx_udp_dest_port),
        .m_udp_length                       (ctrl_tx_udp_length),
        .m_udp_checksum                     (ctrl_tx_udp_checksum),
        .m_udp_payload_axis_tdata           (ctrl_tx_udp_payload_axis_tdata),
        .m_udp_payload_axis_tvalid          (ctrl_tx_udp_payload_axis_tvalid),
        .m_udp_payload_axis_tready          (ctrl_tx_udp_payload_axis_tready),
        .m_udp_payload_axis_tlast           (ctrl_tx_udp_payload_axis_tlast),
        .m_udp_payload_axis_tuser           (ctrl_tx_udp_payload_axis_tuser),

        .m_apb_paddr                        (m_apb_paddr),
        .m_apb_psel                         (m_apb_psel),
        .m_apb_penable                      (m_apb_penable),
        .m_apb_pwrite                       (m_apb_pwrite),
        .m_apb_pwdata                       (m_apb_pwdata),
        .m_apb_pstrb                        (m_apb_pstrb),
        .m_apb_pready                       (m_apb_pready),
        .m_apb_prdata                       (m_apb_prdata),
        .m_apb_pslverr                      (m_apb_pslverr)
    );


generate
    if (DATA_WIDTH == 64) begin
        udp_ip_tx_64 u_control_wrapper_udp_ip_tx_64 (
            .clk                                (clk),
            .rst                                (rst),

            .s_udp_hdr_valid                    (ctrl_tx_udp_hdr_valid),
            .s_udp_hdr_ready                    (ctrl_tx_udp_hdr_ready),
            .s_eth_dest_mac                     (ctrl_tx_eth_dest_mac),
            .s_eth_src_mac                      (ctrl_tx_eth_src_mac),
            .s_eth_type                         (ctrl_tx_eth_type),
            .s_ip_version                       (ctrl_tx_ip_version),
            .s_ip_ihl                           (ctrl_tx_ip_ihl),
            .s_ip_dscp                          (ctrl_tx_ip_dscp),
            .s_ip_ecn                           (ctrl_tx_ip_ecn),
            .s_ip_identification                (ctrl_tx_ip_identification),
            .s_ip_flags                         (ctrl_tx_ip_flags),
            .s_ip_fragment_offset               (ctrl_tx_ip_fragment_offset),
            .s_ip_ttl                           (ctrl_tx_ip_ttl),
            .s_ip_protocol                      (ctrl_tx_ip_protocol),
            .s_ip_header_checksum               (ctrl_tx_ip_header_checksum),
            .s_ip_source_ip                     (ctrl_tx_ip_source_ip),
            .s_ip_dest_ip                       (ctrl_tx_ip_dest_ip),
            .s_udp_source_port                  (ctrl_tx_udp_source_port),
            .s_udp_dest_port                    (ctrl_tx_udp_dest_port),
            .s_udp_length                       (ctrl_tx_udp_length),
            .s_udp_checksum                     (ctrl_tx_udp_checksum),
            .s_udp_payload_axis_tdata           (ctrl_tx_udp_payload_axis_tdata),
            .s_udp_payload_axis_tkeep           (ctrl_tx_udp_payload_axis_tkeep),
            .s_udp_payload_axis_tvalid          (ctrl_tx_udp_payload_axis_tvalid),
            .s_udp_payload_axis_tready          (ctrl_tx_udp_payload_axis_tready),
            .s_udp_payload_axis_tlast           (ctrl_tx_udp_payload_axis_tlast),
            .s_udp_payload_axis_tuser           (ctrl_tx_udp_payload_axis_tuser),

            .m_ip_hdr_valid                     (m_ip_hdr_valid),
            .m_ip_hdr_ready                     (m_ip_hdr_ready),
            .m_eth_dest_mac                     (m_eth_dest_mac),
            .m_eth_src_mac                      (m_eth_src_mac),
            .m_eth_type                         (m_eth_type),
            .m_ip_version                       (m_ip_version),
            .m_ip_ihl                           (m_ip_ihl),
            .m_ip_dscp                          (m_ip_dscp),
            .m_ip_ecn                           (m_ip_ecn),
            .m_ip_length                        (m_ip_length),
            .m_ip_identification                (m_ip_identification),
            .m_ip_flags                         (m_ip_flags),
            .m_ip_fragment_offset               (m_ip_fragment_offset),
            .m_ip_ttl                           (m_ip_ttl),
            .m_ip_protocol                      (m_ip_protocol),
            .m_ip_header_checksum               (m_ip_header_checksum),
            .m_ip_source_ip                     (m_ip_source_ip),
            .m_ip_dest_ip                       (m_ip_dest_ip),
            .m_ip_payload_axis_tdata            (m_ip_payload_axis_tdata),
            .m_ip_payload_axis_tkeep            (m_ip_payload_axis_tkeep),
            .m_ip_payload_axis_tvalid           (m_ip_payload_axis_tvalid),
            .m_ip_payload_axis_tready           (m_ip_payload_axis_tready),
            .m_ip_payload_axis_tlast            (m_ip_payload_axis_tlast),
            .m_ip_payload_axis_tuser            (m_ip_payload_axis_tuser),

            .busy                               (ctrl_tx_busy),
            .error_payload_early_termination    (ctrl_tx_error_payload_early_termination)
        );
    end else if (DATA_WIDTH == 8) begin
        udp_ip_tx u_control_wrapper_udp_ip_tx (
            .clk                                (clk),
            .rst                                (rst),

            .s_udp_hdr_valid                    (ctrl_tx_udp_hdr_valid),
            .s_udp_hdr_ready                    (ctrl_tx_udp_hdr_ready),
            .s_eth_dest_mac                     (ctrl_tx_eth_dest_mac),
            .s_eth_src_mac                      (ctrl_tx_eth_src_mac),
            .s_eth_type                         (ctrl_tx_eth_type),
            .s_ip_version                       (ctrl_tx_ip_version),
            .s_ip_ihl                           (ctrl_tx_ip_ihl),
            .s_ip_dscp                          (ctrl_tx_ip_dscp),
            .s_ip_ecn                           (ctrl_tx_ip_ecn),
            .s_ip_identification                (ctrl_tx_ip_identification),
            .s_ip_flags                         (ctrl_tx_ip_flags),
            .s_ip_fragment_offset               (ctrl_tx_ip_fragment_offset),
            .s_ip_ttl                           (ctrl_tx_ip_ttl),
            .s_ip_protocol                      (ctrl_tx_ip_protocol),
            .s_ip_header_checksum               (ctrl_tx_ip_header_checksum),
            .s_ip_source_ip                     (ctrl_tx_ip_source_ip),
            .s_ip_dest_ip                       (ctrl_tx_ip_dest_ip),
            .s_udp_source_port                  (ctrl_tx_udp_source_port),
            .s_udp_dest_port                    (ctrl_tx_udp_dest_port),
            .s_udp_length                       (ctrl_tx_udp_length),
            .s_udp_checksum                     (ctrl_tx_udp_checksum),
            .s_udp_payload_axis_tdata           (ctrl_tx_udp_payload_axis_tdata),
            .s_udp_payload_axis_tvalid          (ctrl_tx_udp_payload_axis_tvalid),
            .s_udp_payload_axis_tready          (ctrl_tx_udp_payload_axis_tready),
            .s_udp_payload_axis_tlast           (ctrl_tx_udp_payload_axis_tlast),
            .s_udp_payload_axis_tuser           (ctrl_tx_udp_payload_axis_tuser),

            .m_ip_hdr_valid                     (m_ip_hdr_valid),
            .m_ip_hdr_ready                     (m_ip_hdr_ready),
            .m_eth_dest_mac                     (m_eth_dest_mac),
            .m_eth_src_mac                      (m_eth_src_mac),
            .m_eth_type                         (m_eth_type),
            .m_ip_version                       (m_ip_version),
            .m_ip_ihl                           (m_ip_ihl),
            .m_ip_dscp                          (m_ip_dscp),
            .m_ip_ecn                           (m_ip_ecn),
            .m_ip_length                        (m_ip_length),
            .m_ip_identification                (m_ip_identification),
            .m_ip_flags                         (m_ip_flags),
            .m_ip_fragment_offset               (m_ip_fragment_offset),
            .m_ip_ttl                           (m_ip_ttl),
            .m_ip_protocol                      (m_ip_protocol),
            .m_ip_header_checksum               (m_ip_header_checksum),
            .m_ip_source_ip                     (m_ip_source_ip),
            .m_ip_dest_ip                       (m_ip_dest_ip),
            .m_ip_payload_axis_tdata            (m_ip_payload_axis_tdata),
            .m_ip_payload_axis_tvalid           (m_ip_payload_axis_tvalid),
            .m_ip_payload_axis_tready           (m_ip_payload_axis_tready),
            .m_ip_payload_axis_tlast            (m_ip_payload_axis_tlast),
            .m_ip_payload_axis_tuser            (m_ip_payload_axis_tuser),

            .busy                               (ctrl_tx_busy),
            .error_payload_early_termination    (ctrl_tx_error_payload_early_termination)
        );

        assign ctrl_tx_udp_payload_axis_tkeep = ctrl_tx_udp_payload_axis_tvalid;
    end else begin
        $error("Only 8 and 64 bit data widths are supported");
    end
endgenerate

endmodule