module l3 #(
    parameter DATA_WIDTH = 64,
    parameter KEEP_ENABLE = (DATA_WIDTH>8),
    parameter KEEP_WIDTH = (DATA_WIDTH/8),
    parameter USER_WIDTH = 1
)
(
    input  wire                          clk,
    input  wire                          rst,

    /*
     * Ethernet frame input
     */
    input  wire                          s_eth_hdr_valid,
    output wire                          s_eth_hdr_ready,
    input  wire [47:0]                   s_eth_dest_mac,
    input  wire [47:0]                   s_eth_src_mac,
    input  wire [15:0]                   s_eth_type,
    input  wire [DATA_WIDTH-1:0]         s_eth_payload_axis_tdata,
    input  wire [KEEP_WIDTH-1:0]         s_eth_payload_axis_tkeep,
    input  wire                          s_eth_payload_axis_tvalid,
    output wire                          s_eth_payload_axis_tready,
    input  wire                          s_eth_payload_axis_tlast,
    input  wire [USER_WIDTH-1:0]         s_eth_payload_axis_tuser,

    /*
     * Ethernet frame output
     */
    output wire                          m_eth_hdr_valid,
    input  wire                          m_eth_hdr_ready,
    output wire [47:0]                   m_eth_dest_mac,
    output wire [47:0]                   m_eth_src_mac,
    output wire [15:0]                   m_eth_type,
    output wire [DATA_WIDTH-1:0]         m_eth_payload_axis_tdata,
    output wire [KEEP_WIDTH-1:0]         m_eth_payload_axis_tkeep,
    output wire                          m_eth_payload_axis_tvalid,
    input  wire                          m_eth_payload_axis_tready,
    output wire                          m_eth_payload_axis_tlast,
    output wire [USER_WIDTH-1:0]         m_eth_payload_axis_tuser,

    // APB Master
    output wire [31:0]                  m_apb_paddr,
    output wire                         m_apb_psel,
    output wire                         m_apb_penable,
    output wire                         m_apb_pwrite,
    output wire [31:0]                  m_apb_pwdata,
    output wire [3:0]                   m_apb_pstrb,
    input  wire                         m_apb_pready,
    input  wire [31:0]                  m_apb_prdata,
    input  wire                         m_apb_pslverr
);

    `define IP_PROTO_ICMP 8'h1
    `define IP_PROTO_UDP 8'h11

    localparam ARP_ETHERTYPE = 16'h0806;

    localparam LOCAL_MAC = 48'h020000aabbcc;
    localparam LOCAL_IP  = 32'hac000002;      // should be a register setting
    localparam GATEWAY_IP = 32'hac000001;
    localparam SUBNET_MASK = 32'hffffff00;


    logic                           demux_ip_eth_hdr_valid;
    logic                           demux_ip_eth_hdr_ready;
    logic   [47:0]                  demux_ip_eth_dest_mac;
    logic   [47:0]                  demux_ip_eth_src_mac;
    logic   [15:0]                  demux_ip_eth_type;
    logic   [DATA_WIDTH-1:0]        demux_ip_eth_payload_axis_tdata;
    logic   [KEEP_WIDTH-1:0]        demux_ip_eth_payload_axis_tkeep;
    logic                           demux_ip_eth_payload_axis_tvalid;
    logic                           demux_ip_eth_payload_axis_tready;
    logic                           demux_ip_eth_payload_axis_tlast;
    logic                           demux_ip_eth_payload_axis_tuser;

    logic                           demux_arp_eth_hdr_valid;
    logic                           demux_arp_eth_hdr_ready;
    logic   [47:0]                  demux_arp_eth_dest_mac;
    logic   [47:0]                  demux_arp_eth_src_mac;
    logic   [15:0]                  demux_arp_eth_type;
    logic   [DATA_WIDTH-1:0]        demux_arp_eth_payload_axis_tdata;
    logic   [KEEP_WIDTH-1:0]        demux_arp_eth_payload_axis_tkeep;
    logic                           demux_arp_eth_payload_axis_tvalid;
    logic                           demux_arp_eth_payload_axis_tready;
    logic                           demux_arp_eth_payload_axis_tlast;
    logic                           demux_arp_eth_payload_axis_tuser;

    logic                           rx_ip_hdr_valid;
    logic                           rx_ip_hdr_ready;
    logic   [47:0]                  rx_eth_dest_mac;
    logic   [47:0]                  rx_eth_src_mac;
    logic   [15:0]                  rx_eth_type;
    logic   [3:0]                   rx_ip_version;
    logic   [3:0]                   rx_ip_ihl;
    logic   [5:0]                   rx_ip_dscp;
    logic   [1:0]                   rx_ip_ecn;
    logic   [15:0]                  rx_ip_length;
    logic   [15:0]                  rx_ip_identification;
    logic   [2:0]                   rx_ip_flags;
    logic   [12:0]                  rx_ip_fragment_offset;
    logic   [7:0]                   rx_ip_ttl;
    logic   [7:0]                   rx_ip_protocol;
    logic   [15:0]                  rx_ip_header_checksum;
    logic   [31:0]                  rx_ip_source_ip;
    logic   [31:0]                  rx_ip_dest_ip;
    logic   [DATA_WIDTH-1:0]        rx_ip_payload_axis_tdata;
    logic   [KEEP_WIDTH-1:0]        rx_ip_payload_axis_tkeep;
    logic                           rx_ip_payload_axis_tvalid;
    logic                           rx_ip_payload_axis_tready;
    logic                           rx_ip_payload_axis_tlast;
    logic                           rx_ip_payload_axis_tuser;


    logic                           rx_busy;
    logic                           rx_error_header_early_termination;
    logic                           rx_error_payload_early_termination;
    logic                           rx_error_invalid_header;
    logic                           rx_error_invalid_checksum;

    logic                           ctrl_rx_ip_hdr_valid;
    logic                           ctrl_rx_ip_hdr_ready;
    logic   [47:0]                  ctrl_rx_eth_dest_mac;
    logic   [47:0]                  ctrl_rx_eth_src_mac;
    logic   [15:0]                  ctrl_rx_eth_type;
    logic   [3:0]                   ctrl_rx_ip_version;
    logic   [3:0]                   ctrl_rx_ip_ihl;
    logic   [5:0]                   ctrl_rx_ip_dscp;
    logic   [1:0]                   ctrl_rx_ip_ecn;
    logic   [15:0]                  ctrl_rx_ip_length;
    logic   [15:0]                  ctrl_rx_ip_identification;
    logic   [2:0]                   ctrl_rx_ip_flags;
    logic   [12:0]                  ctrl_rx_ip_fragment_offset;
    logic   [7:0]                   ctrl_rx_ip_ttl;
    logic   [7:0]                   ctrl_rx_ip_protocol;
    logic   [15:0]                  ctrl_rx_ip_header_checksum;
    logic   [31:0]                  ctrl_rx_ip_source_ip;
    logic   [31:0]                  ctrl_rx_ip_dest_ip;
    logic   [DATA_WIDTH-1:0]        ctrl_rx_ip_payload_axis_tdata;
    logic   [KEEP_WIDTH-1:0]        ctrl_rx_ip_payload_axis_tkeep;
    logic                           ctrl_rx_ip_payload_axis_tvalid;
    logic                           ctrl_rx_ip_payload_axis_tready;
    logic                           ctrl_rx_ip_payload_axis_tlast;
    logic                           ctrl_rx_ip_payload_axis_tuser;

    logic                           ctrl_tx_ip_hdr_valid;
    logic                           ctrl_tx_ip_hdr_ready;
    logic   [47:0]                  ctrl_tx_eth_dest_mac;
    logic   [47:0]                  ctrl_tx_eth_src_mac;
    logic   [15:0]                  ctrl_tx_eth_type;
    logic   [3:0]                   ctrl_tx_ip_version;
    logic   [3:0]                   ctrl_tx_ip_ihl;
    logic   [5:0]                   ctrl_tx_ip_dscp;
    logic   [1:0]                   ctrl_tx_ip_ecn;
    logic   [15:0]                  ctrl_tx_ip_length;
    logic   [15:0]                  ctrl_tx_ip_identification;
    logic   [2:0]                   ctrl_tx_ip_flags;
    logic   [12:0]                  ctrl_tx_ip_fragment_offset;
    logic   [7:0]                   ctrl_tx_ip_ttl;
    logic   [7:0]                   ctrl_tx_ip_protocol;
    logic   [15:0]                  ctrl_tx_ip_header_checksum;
    logic   [31:0]                  ctrl_tx_ip_source_ip;
    logic   [31:0]                  ctrl_tx_ip_dest_ip;
    logic   [DATA_WIDTH-1:0]        ctrl_tx_ip_payload_axis_tdata;
    logic   [KEEP_WIDTH-1:0]        ctrl_tx_ip_payload_axis_tkeep;
    logic                           ctrl_tx_ip_payload_axis_tvalid;
    logic                           ctrl_tx_ip_payload_axis_tready;
    logic                           ctrl_tx_ip_payload_axis_tlast;
    logic                           ctrl_tx_ip_payload_axis_tuser;

    logic                           ping_rx_ip_hdr_valid;
    logic                           ping_rx_ip_hdr_ready;
    logic   [47:0]                  ping_rx_eth_dest_mac;
    logic   [47:0]                  ping_rx_eth_src_mac;
    logic   [15:0]                  ping_rx_eth_type;
    logic   [3:0]                   ping_rx_ip_version;
    logic   [3:0]                   ping_rx_ip_ihl;
    logic   [5:0]                   ping_rx_ip_dscp;
    logic   [1:0]                   ping_rx_ip_ecn;
    logic   [15:0]                  ping_rx_ip_length;
    logic   [15:0]                  ping_rx_ip_identification;
    logic   [2:0]                   ping_rx_ip_flags;
    logic   [12:0]                  ping_rx_ip_fragment_offset;
    logic   [7:0]                   ping_rx_ip_ttl;
    logic   [7:0]                   ping_rx_ip_protocol;
    logic   [15:0]                  ping_rx_ip_header_checksum;
    logic   [31:0]                  ping_rx_ip_source_ip;
    logic   [31:0]                  ping_rx_ip_dest_ip;
    logic   [DATA_WIDTH-1:0]        ping_rx_ip_payload_axis_tdata;
    logic   [KEEP_WIDTH-1:0]        ping_rx_ip_payload_axis_tkeep;
    logic                           ping_rx_ip_payload_axis_tvalid;
    logic                           ping_rx_ip_payload_axis_tready;
    logic                           ping_rx_ip_payload_axis_tlast;
    logic                           ping_rx_ip_payload_axis_tuser;

    logic                           rx_ip_demux_sel;
    logic                           rx_ip_demux_drop;

    logic                           ping_tx_ip_hdr_valid;
    logic                           ping_tx_ip_hdr_ready;
    logic   [47:0]                  ping_tx_eth_dest_mac;
    logic   [47:0]                  ping_tx_eth_src_mac;
    logic   [15:0]                  ping_tx_eth_type;
    logic   [3:0]                   ping_tx_ip_version;
    logic   [3:0]                   ping_tx_ip_ihl;
    logic   [5:0]                   ping_tx_ip_dscp;
    logic   [1:0]                   ping_tx_ip_ecn;
    logic   [15:0]                  ping_tx_ip_length;
    logic   [15:0]                  ping_tx_ip_identification;
    logic   [2:0]                   ping_tx_ip_flags;
    logic   [12:0]                  ping_tx_ip_fragment_offset;
    logic   [7:0]                   ping_tx_ip_ttl;
    logic   [7:0]                   ping_tx_ip_protocol;
    logic   [15:0]                  ping_tx_ip_header_checksum;
    logic   [31:0]                  ping_tx_ip_source_ip;
    logic   [31:0]                  ping_tx_ip_dest_ip;
    logic   [DATA_WIDTH-1:0]        ping_tx_ip_payload_axis_tdata;
    logic   [KEEP_WIDTH-1:0]        ping_tx_ip_payload_axis_tkeep;
    logic                           ping_tx_ip_payload_axis_tvalid;
    logic                           ping_tx_ip_payload_axis_tready;
    logic                           ping_tx_ip_payload_axis_tlast;
    logic                           ping_tx_ip_payload_axis_tuser;

    logic                           tx_ip_hdr_valid;
    logic                           tx_ip_hdr_ready;
    logic   [47:0]                  tx_eth_dest_mac;
    logic   [47:0]                  tx_eth_src_mac;
    logic   [15:0]                  tx_eth_type;
    logic   [3:0]                   tx_ip_version;
    logic   [3:0]                   tx_ip_ihl;
    logic   [5:0]                   tx_ip_dscp;
    logic   [1:0]                   tx_ip_ecn;
    logic   [15:0]                  tx_ip_length;
    logic   [15:0]                  tx_ip_identification;
    logic   [2:0]                   tx_ip_flags;
    logic   [12:0]                  tx_ip_fragment_offset;
    logic   [7:0]                   tx_ip_ttl;
    logic   [7:0]                   tx_ip_protocol;
    logic   [15:0]                  tx_ip_header_checksum;
    logic   [31:0]                  tx_ip_source_ip;
    logic   [31:0]                  tx_ip_dest_ip;
    logic   [DATA_WIDTH-1:0]        tx_ip_payload_axis_tdata;
    logic   [KEEP_WIDTH-1:0]        tx_ip_payload_axis_tkeep;
    logic                           tx_ip_payload_axis_tvalid;
    logic                           tx_ip_payload_axis_tready;
    logic                           tx_ip_payload_axis_tlast;
    logic                           tx_ip_payload_axis_tuser;

    logic                           ip_tx_eth_hdr_valid;
    logic                           ip_tx_eth_hdr_ready;
    logic   [47:0]                  ip_tx_eth_dest_mac;
    logic   [47:0]                  ip_tx_eth_src_mac;
    logic   [15:0]                  ip_tx_eth_type;
    logic   [DATA_WIDTH-1:0]        ip_tx_eth_payload_axis_tdata;
    logic   [KEEP_WIDTH-1:0]        ip_tx_eth_payload_axis_tkeep;
    logic                           ip_tx_eth_payload_axis_tvalid;
    logic                           ip_tx_eth_payload_axis_tready;
    logic                           ip_tx_eth_payload_axis_tlast;
    logic                           ip_tx_eth_payload_axis_tuser;

    logic                           arp_tx_eth_hdr_valid;
    logic                           arp_tx_eth_hdr_ready;
    logic   [47:0]                  arp_tx_eth_dest_mac;
    logic   [47:0]                  arp_tx_eth_src_mac;
    logic   [15:0]                  arp_tx_eth_type;
    logic   [DATA_WIDTH-1:0]        arp_tx_eth_payload_axis_tdata;
    logic   [KEEP_WIDTH-1:0]        arp_tx_eth_payload_axis_tkeep;
    logic                           arp_tx_eth_payload_axis_tvalid;
    logic                           arp_tx_eth_payload_axis_tready;
    logic                           arp_tx_eth_payload_axis_tlast;
    logic                           arp_tx_eth_payload_axis_tuser;

    logic           tx_busy;
    logic           tx_error_payload_early_termination;


    assign eth_rx_demux_sel = (s_eth_type == ARP_ETHERTYPE) ? 0 : 1;

    eth_demux #(
        .M_COUNT        (2),
        .DATA_WIDTH     (DATA_WIDTH),
        .ID_ENABLE      (0),
        .DEST_ENABLE    (0)
    ) u_ntw_eth_rx_demux (
        .clk                                (clk),
        .rst                                (rst),

        .s_eth_hdr_valid                    (s_eth_hdr_valid),
        .s_eth_hdr_ready                    (s_eth_hdr_ready),
        .s_eth_dest_mac                     (s_eth_dest_mac),
        .s_eth_src_mac                      (s_eth_src_mac),
        .s_eth_type                         (s_eth_type),
        .s_eth_payload_axis_tdata           (s_eth_payload_axis_tdata),
        .s_eth_payload_axis_tkeep           (s_eth_payload_axis_tkeep),
        .s_eth_payload_axis_tvalid          (s_eth_payload_axis_tvalid),
        .s_eth_payload_axis_tready          (s_eth_payload_axis_tready),
        .s_eth_payload_axis_tlast           (s_eth_payload_axis_tlast),
        .s_eth_payload_axis_tid             ('0),
        .s_eth_payload_axis_tdest           ('0),
        .s_eth_payload_axis_tuser           (s_eth_payload_axis_tuser),

        .m_eth_hdr_valid                    ({demux_ip_eth_hdr_valid,            demux_arp_eth_hdr_valid             }),
        .m_eth_hdr_ready                    ({demux_ip_eth_hdr_ready,            demux_arp_eth_hdr_ready             }),
        .m_eth_dest_mac                     ({demux_ip_eth_dest_mac,             demux_arp_eth_dest_mac              }),
        .m_eth_src_mac                      ({demux_ip_eth_src_mac,              demux_arp_eth_src_mac               }),
        .m_eth_type                         ({demux_ip_eth_type,                 demux_arp_eth_type                  }),
        .m_eth_payload_axis_tdata           ({demux_ip_eth_payload_axis_tdata,   demux_arp_eth_payload_axis_tdata    }),
        .m_eth_payload_axis_tkeep           ({demux_ip_eth_payload_axis_tkeep,   demux_arp_eth_payload_axis_tkeep    }),
        .m_eth_payload_axis_tvalid          ({demux_ip_eth_payload_axis_tvalid,  demux_arp_eth_payload_axis_tvalid   }),
        .m_eth_payload_axis_tready          ({demux_ip_eth_payload_axis_tready,  demux_arp_eth_payload_axis_tready   }),
        .m_eth_payload_axis_tlast           ({demux_ip_eth_payload_axis_tlast,   demux_arp_eth_payload_axis_tlast    }),
        .m_eth_payload_axis_tid             (),
        .m_eth_payload_axis_tdest           (),
        .m_eth_payload_axis_tuser           ({demux_ip_eth_payload_axis_tuser,   demux_arp_eth_payload_axis_tuser    }),

        .enable                             ('1),
        .drop                               ('0), //?
        .select                             (eth_rx_demux_sel)
    );

    arp #(
        .DATA_WIDTH                         (DATA_WIDTH),
        .CACHE_ADDR_WIDTH                   (2)
    ) u_arp (
        .clk                                (clk),
        .rst                                (rst),

        .s_eth_hdr_valid                    (demux_arp_eth_hdr_valid),
        .s_eth_hdr_ready                    (demux_arp_eth_hdr_ready),
        .s_eth_dest_mac                     (demux_arp_eth_dest_mac),
        .s_eth_src_mac                      (demux_arp_eth_src_mac),
        .s_eth_type                         (demux_arp_eth_type),
        .s_eth_payload_axis_tdata           (demux_arp_eth_payload_axis_tdata),
        .s_eth_payload_axis_tkeep           (demux_arp_eth_payload_axis_tkeep),
        .s_eth_payload_axis_tvalid          (demux_arp_eth_payload_axis_tvalid),
        .s_eth_payload_axis_tready          (demux_arp_eth_payload_axis_tready),
        .s_eth_payload_axis_tlast           (demux_arp_eth_payload_axis_tlast),
        .s_eth_payload_axis_tuser           (demux_arp_eth_payload_axis_tuser),

        .m_eth_hdr_valid                    (arp_tx_eth_hdr_valid),
        .m_eth_hdr_ready                    (arp_tx_eth_hdr_ready),
        .m_eth_dest_mac                     (arp_tx_eth_dest_mac),
        .m_eth_src_mac                      (arp_tx_eth_src_mac),
        .m_eth_type                         (arp_tx_eth_type),
        .m_eth_payload_axis_tdata           (arp_tx_eth_payload_axis_tdata),
        .m_eth_payload_axis_tkeep           (arp_tx_eth_payload_axis_tkeep),
        .m_eth_payload_axis_tvalid          (arp_tx_eth_payload_axis_tvalid),
        .m_eth_payload_axis_tready          (arp_tx_eth_payload_axis_tready),
        .m_eth_payload_axis_tlast           (arp_tx_eth_payload_axis_tlast),
        .m_eth_payload_axis_tuser           (arp_tx_eth_payload_axis_tuser),

        .arp_request_valid                  ('0),
        .arp_request_ready                  (),
        .arp_request_ip                     (),
        .arp_response_valid                 (),
        .arp_response_ready                 ('0),
        .arp_response_error                 (),
        .arp_response_mac                   (),

        .local_mac                          (LOCAL_MAC),
        .local_ip                           (LOCAL_IP),
        .gateway_ip                         (GATEWAY_IP),
        .subnet_mask                        (SUBNET_MASK),
        .clear_cache                        ('0)
    );

generate
    if (DATA_WIDTH == 64) begin
        ip_eth_rx_64 u_l3_ip_eth_rx_64 (
            .clk                                (clk),
            .rst                                (rst),

            .s_eth_hdr_valid                    (demux_ip_eth_hdr_valid),
            .s_eth_hdr_ready                    (demux_ip_eth_hdr_ready),
            .s_eth_dest_mac                     (demux_ip_eth_dest_mac),
            .s_eth_src_mac                      (demux_ip_eth_src_mac),
            .s_eth_type                         (demux_ip_eth_type),
            .s_eth_payload_axis_tdata           (demux_ip_eth_payload_axis_tdata),
            .s_eth_payload_axis_tkeep           (demux_ip_eth_payload_axis_tkeep),
            .s_eth_payload_axis_tvalid          (demux_ip_eth_payload_axis_tvalid),
            .s_eth_payload_axis_tready          (demux_ip_eth_payload_axis_tready),
            .s_eth_payload_axis_tlast           (demux_ip_eth_payload_axis_tlast),
            .s_eth_payload_axis_tuser           (demux_ip_eth_payload_axis_tuser),

            .m_ip_hdr_valid                     (rx_ip_hdr_valid),
            .m_ip_hdr_ready                     (rx_ip_hdr_ready),
            .m_eth_dest_mac                     (rx_eth_dest_mac),
            .m_eth_src_mac                      (rx_eth_src_mac),
            .m_eth_type                         (rx_eth_type),
            .m_ip_version                       (rx_ip_version),
            .m_ip_ihl                           (rx_ip_ihl),
            .m_ip_dscp                          (rx_ip_dscp),
            .m_ip_ecn                           (rx_ip_ecn),
            .m_ip_length                        (rx_ip_length),
            .m_ip_identification                (rx_ip_identification),
            .m_ip_flags                         (rx_ip_flags),
            .m_ip_fragment_offset               (rx_ip_fragment_offset),
            .m_ip_ttl                           (rx_ip_ttl),
            .m_ip_protocol                      (rx_ip_protocol),
            .m_ip_header_checksum               (rx_ip_header_checksum),
            .m_ip_source_ip                     (rx_ip_source_ip),
            .m_ip_dest_ip                       (rx_ip_dest_ip),
            .m_ip_payload_axis_tdata            (rx_ip_payload_axis_tdata),
            .m_ip_payload_axis_tkeep            (rx_ip_payload_axis_tkeep),
            .m_ip_payload_axis_tvalid           (rx_ip_payload_axis_tvalid),
            .m_ip_payload_axis_tready           (rx_ip_payload_axis_tready),
            .m_ip_payload_axis_tlast            (rx_ip_payload_axis_tlast),
            .m_ip_payload_axis_tuser            (rx_ip_payload_axis_tuser),

            .busy                               (rx_busy),
            .error_header_early_termination     (rx_error_header_early_termination),
            .error_payload_early_termination    (rx_error_payload_early_termination),
            .error_invalid_header               (rx_error_invalid_header),
            .error_invalid_checksum             (rx_error_invalid_checksum)
        );
    end else if (DATA_WIDTH == 8) begin
        ip_eth_rx u_l3_ip_eth_rx (
            .clk                                (clk),
            .rst                                (rst),

            .s_eth_hdr_valid                    (demux_ip_eth_hdr_valid),
            .s_eth_hdr_ready                    (demux_ip_eth_hdr_ready),
            .s_eth_dest_mac                     (demux_ip_eth_dest_mac),
            .s_eth_src_mac                      (demux_ip_eth_src_mac),
            .s_eth_type                         (demux_ip_eth_type),
            .s_eth_payload_axis_tdata           (demux_ip_eth_payload_axis_tdata),
            .s_eth_payload_axis_tvalid          (demux_ip_eth_payload_axis_tvalid),
            .s_eth_payload_axis_tready          (demux_ip_eth_payload_axis_tready),
            .s_eth_payload_axis_tlast           (demux_ip_eth_payload_axis_tlast),
            .s_eth_payload_axis_tuser           (demux_ip_eth_payload_axis_tuser),

            .m_ip_hdr_valid                     (rx_ip_hdr_valid),
            .m_ip_hdr_ready                     (rx_ip_hdr_ready),
            .m_eth_dest_mac                     (rx_eth_dest_mac),
            .m_eth_src_mac                      (rx_eth_src_mac),
            .m_eth_type                         (rx_eth_type),
            .m_ip_version                       (rx_ip_version),
            .m_ip_ihl                           (rx_ip_ihl),
            .m_ip_dscp                          (rx_ip_dscp),
            .m_ip_ecn                           (rx_ip_ecn),
            .m_ip_length                        (rx_ip_length),
            .m_ip_identification                (rx_ip_identification),
            .m_ip_flags                         (rx_ip_flags),
            .m_ip_fragment_offset               (rx_ip_fragment_offset),
            .m_ip_ttl                           (rx_ip_ttl),
            .m_ip_protocol                      (rx_ip_protocol),
            .m_ip_header_checksum               (rx_ip_header_checksum),
            .m_ip_source_ip                     (rx_ip_source_ip),
            .m_ip_dest_ip                       (rx_ip_dest_ip),
            .m_ip_payload_axis_tdata            (rx_ip_payload_axis_tdata),
            .m_ip_payload_axis_tvalid           (rx_ip_payload_axis_tvalid),
            .m_ip_payload_axis_tready           (rx_ip_payload_axis_tready),
            .m_ip_payload_axis_tlast            (rx_ip_payload_axis_tlast),
            .m_ip_payload_axis_tuser            (rx_ip_payload_axis_tuser),

            .busy                               (rx_busy),
            .error_header_early_termination     (rx_error_header_early_termination),
            .error_payload_early_termination    (rx_error_payload_early_termination),
            .error_invalid_header               (rx_error_invalid_header),
            .error_invalid_checksum             (rx_error_invalid_checksum)
        );

        assign rx_ip_payload_axis_tkeep = rx_ip_payload_axis_tvalid;
    end else begin
        $error("Only 8 and 64 bit data widths are supported");
    end
endgenerate

    assign rx_ip_demux_sel = (rx_ip_protocol == `IP_PROTO_ICMP) ? 0 : 1;
    assign rx_ip_demux_drop = (rx_ip_protocol != `IP_PROTO_ICMP && rx_ip_protocol != `IP_PROTO_UDP) ? 1 : 0;

    ip_demux  #(
        .M_COUNT(2),
        .DATA_WIDTH(DATA_WIDTH)
    ) u_l3_rx_ip_demux (
        .clk                                (clk),
        .rst                                (rst),

        .s_ip_hdr_valid                     (rx_ip_hdr_valid),
        .s_ip_hdr_ready                     (rx_ip_hdr_ready),
        .s_eth_dest_mac                     (rx_eth_dest_mac),
        .s_eth_src_mac                      (rx_eth_src_mac),
        .s_eth_type                         (rx_eth_type),
        .s_ip_version                       (rx_ip_version),
        .s_ip_ihl                           (rx_ip_ihl),
        .s_ip_dscp                          (rx_ip_dscp),
        .s_ip_ecn                           (rx_ip_ecn),
        .s_ip_length                        (rx_ip_length),
        .s_ip_identification                (rx_ip_identification),
        .s_ip_flags                         (rx_ip_flags),
        .s_ip_fragment_offset               (rx_ip_fragment_offset),
        .s_ip_ttl                           (rx_ip_ttl),
        .s_ip_protocol                      (rx_ip_protocol),
        .s_ip_header_checksum               (rx_ip_header_checksum),
        .s_ip_source_ip                     (rx_ip_source_ip),
        .s_ip_dest_ip                       (rx_ip_dest_ip),
        .s_ip_payload_axis_tdata            (rx_ip_payload_axis_tdata),
        .s_ip_payload_axis_tkeep            (rx_ip_payload_axis_tkeep),
        .s_ip_payload_axis_tvalid           (rx_ip_payload_axis_tvalid),
        .s_ip_payload_axis_tready           (rx_ip_payload_axis_tready),
        .s_ip_payload_axis_tlast            (rx_ip_payload_axis_tlast),
        .s_ip_payload_axis_tid              ('0),
        .s_ip_payload_axis_tdest            ('0),
        .s_ip_payload_axis_tuser            (rx_ip_payload_axis_tuser),

        .m_ip_hdr_valid                     ({ctrl_rx_ip_hdr_valid,             ping_rx_ip_hdr_valid            }),
        .m_ip_hdr_ready                     ({ctrl_rx_ip_hdr_ready,             ping_rx_ip_hdr_ready            }),
        .m_eth_dest_mac                     ({ctrl_rx_eth_dest_mac,             ping_rx_eth_dest_mac            }),
        .m_eth_src_mac                      ({ctrl_rx_eth_src_mac,              ping_rx_eth_src_mac             }),
        .m_eth_type                         ({ctrl_rx_eth_type,                 ping_rx_eth_type                }),
        .m_ip_version                       ({ctrl_rx_ip_version,               ping_rx_ip_version              }),
        .m_ip_ihl                           ({ctrl_rx_ip_ihl,                   ping_rx_ip_ihl                  }),
        .m_ip_dscp                          ({ctrl_rx_ip_dscp,                  ping_rx_ip_dscp                 }),
        .m_ip_ecn                           ({ctrl_rx_ip_ecn,                   ping_rx_ip_ecn                  }),
        .m_ip_length                        ({ctrl_rx_ip_length,                ping_rx_ip_length               }),
        .m_ip_identification                ({ctrl_rx_ip_identification,        ping_rx_ip_identification       }),
        .m_ip_flags                         ({ctrl_rx_ip_flags,                 ping_rx_ip_flags                }),
        .m_ip_fragment_offset               ({ctrl_rx_ip_fragment_offset,       ping_rx_ip_fragment_offset      }),
        .m_ip_ttl                           ({ctrl_rx_ip_ttl,                   ping_rx_ip_ttl                  }),
        .m_ip_protocol                      ({ctrl_rx_ip_protocol,              ping_rx_ip_protocol             }),
        .m_ip_header_checksum               ({ctrl_rx_ip_header_checksum,       ping_rx_ip_header_checksum      }),
        .m_ip_source_ip                     ({ctrl_rx_ip_source_ip,             ping_rx_ip_source_ip            }),
        .m_ip_dest_ip                       ({ctrl_rx_ip_dest_ip,               ping_rx_ip_dest_ip              }),
        .m_ip_payload_axis_tdata            ({ctrl_rx_ip_payload_axis_tdata,    ping_rx_ip_payload_axis_tdata   }),
        .m_ip_payload_axis_tkeep            ({ctrl_rx_ip_payload_axis_tkeep,    ping_rx_ip_payload_axis_tkeep   }),
        .m_ip_payload_axis_tvalid           ({ctrl_rx_ip_payload_axis_tvalid,   ping_rx_ip_payload_axis_tvalid  }),
        .m_ip_payload_axis_tready           ({ctrl_rx_ip_payload_axis_tready,   ping_rx_ip_payload_axis_tready  }),
        .m_ip_payload_axis_tlast            ({ctrl_rx_ip_payload_axis_tlast,    ping_rx_ip_payload_axis_tlast   }),
        .m_ip_payload_axis_tid              (),
        .m_ip_payload_axis_tdest            (),
        .m_ip_payload_axis_tuser            ({ctrl_rx_ip_payload_axis_tuser,    ping_rx_ip_payload_axis_tuser   }),

        .enable                             ('1),
        .drop                               (rx_ip_demux_drop),
        .select                             (rx_ip_demux_sel)
    );

    control_wrapper #(
        .DATA_WIDTH(DATA_WIDTH)
    ) u_control_wrapper (
        .clk                                (clk),
        .rst                                (rst),

        .s_ip_hdr_valid                     (ctrl_rx_ip_hdr_valid),
        .s_ip_hdr_ready                     (ctrl_rx_ip_hdr_ready),
        .s_eth_dest_mac                     (ctrl_rx_eth_dest_mac),
        .s_eth_src_mac                      (ctrl_rx_eth_src_mac),
        .s_eth_type                         (ctrl_rx_eth_type),
        .s_ip_version                       (ctrl_rx_ip_version),
        .s_ip_ihl                           (ctrl_rx_ip_ihl),
        .s_ip_dscp                          (ctrl_rx_ip_dscp),
        .s_ip_ecn                           (ctrl_rx_ip_ecn),
        .s_ip_length                        (ctrl_rx_ip_length),
        .s_ip_identification                (ctrl_rx_ip_identification),
        .s_ip_flags                         (ctrl_rx_ip_flags),
        .s_ip_fragment_offset               (ctrl_rx_ip_fragment_offset),
        .s_ip_ttl                           (ctrl_rx_ip_ttl),
        .s_ip_protocol                      (ctrl_rx_ip_protocol),
        .s_ip_header_checksum               (ctrl_rx_ip_header_checksum),
        .s_ip_source_ip                     (ctrl_rx_ip_source_ip),
        .s_ip_dest_ip                       (ctrl_rx_ip_dest_ip),
        .s_ip_payload_axis_tdata            (ctrl_rx_ip_payload_axis_tdata),
        .s_ip_payload_axis_tkeep            (ctrl_rx_ip_payload_axis_tkeep),
        .s_ip_payload_axis_tvalid           (ctrl_rx_ip_payload_axis_tvalid),
        .s_ip_payload_axis_tready           (ctrl_rx_ip_payload_axis_tready),
        .s_ip_payload_axis_tlast            (ctrl_rx_ip_payload_axis_tlast),
        .s_ip_payload_axis_tuser            (ctrl_rx_ip_payload_axis_tuser),

        .m_ip_hdr_valid                     (ctrl_tx_ip_hdr_valid),
        .m_ip_hdr_ready                     (ctrl_tx_ip_hdr_ready),
        .m_eth_dest_mac                     (ctrl_tx_eth_dest_mac),
        .m_eth_src_mac                      (ctrl_tx_eth_src_mac),
        .m_eth_type                         (ctrl_tx_eth_type),
        .m_ip_version                       (ctrl_tx_ip_version),
        .m_ip_ihl                           (ctrl_tx_ip_ihl),
        .m_ip_dscp                          (ctrl_tx_ip_dscp),
        .m_ip_ecn                           (ctrl_tx_ip_ecn),
        .m_ip_length                        (ctrl_tx_ip_length),
        .m_ip_identification                (ctrl_tx_ip_identification),
        .m_ip_flags                         (ctrl_tx_ip_flags),
        .m_ip_fragment_offset               (ctrl_tx_ip_fragment_offset),
        .m_ip_ttl                           (ctrl_tx_ip_ttl),
        .m_ip_protocol                      (ctrl_tx_ip_protocol),
        .m_ip_header_checksum               (ctrl_tx_ip_header_checksum),
        .m_ip_source_ip                     (ctrl_tx_ip_source_ip),
        .m_ip_dest_ip                       (ctrl_tx_ip_dest_ip),
        .m_ip_payload_axis_tdata            (ctrl_tx_ip_payload_axis_tdata),
        .m_ip_payload_axis_tkeep            (ctrl_tx_ip_payload_axis_tkeep),
        .m_ip_payload_axis_tvalid           (ctrl_tx_ip_payload_axis_tvalid),
        .m_ip_payload_axis_tready           (ctrl_tx_ip_payload_axis_tready),
        .m_ip_payload_axis_tlast            (ctrl_tx_ip_payload_axis_tlast),
        .m_ip_payload_axis_tuser            (ctrl_tx_ip_payload_axis_tuser),

        .m_apb_paddr                        (m_apb_paddr),
        .m_apb_psel                         (m_apb_psel),
        .m_apb_penable                      (m_apb_penable),
        .m_apb_pwrite                       (m_apb_pwrite),
        .m_apb_pwdata                       (m_apb_pwdata),
        .m_apb_pstrb                        (m_apb_pstrb),
        .m_apb_pready                       (m_apb_pready),
        .m_apb_prdata                       (m_apb_prdata),
        .m_apb_pslverr                      (m_apb_pslverr)
    );

    ping_wrapper #(
        .DATA_WIDTH(DATA_WIDTH)
    ) u_ping_wrapper (
        .clk                                (clk),
        .rst                                (rst),

        .s_ip_hdr_valid                     (ping_rx_ip_hdr_valid),
        .s_ip_hdr_ready                     (ping_rx_ip_hdr_ready),
        .s_eth_dest_mac                     (ping_rx_eth_dest_mac),
        .s_eth_src_mac                      (ping_rx_eth_src_mac),
        .s_eth_type                         (ping_rx_eth_type),
        .s_ip_version                       (ping_rx_ip_version),
        .s_ip_ihl                           (ping_rx_ip_ihl),
        .s_ip_dscp                          (ping_rx_ip_dscp),
        .s_ip_ecn                           (ping_rx_ip_ecn),
        .s_ip_length                        (ping_rx_ip_length),
        .s_ip_identification                (ping_rx_ip_identification),
        .s_ip_flags                         (ping_rx_ip_flags),
        .s_ip_fragment_offset               (ping_rx_ip_fragment_offset),
        .s_ip_ttl                           (ping_rx_ip_ttl),
        .s_ip_protocol                      (ping_rx_ip_protocol),
        .s_ip_header_checksum               (ping_rx_ip_header_checksum),
        .s_ip_source_ip                     (ping_rx_ip_source_ip),
        .s_ip_dest_ip                       (ping_rx_ip_dest_ip),
        .s_ip_payload_axis_tdata            (ping_rx_ip_payload_axis_tdata),
        .s_ip_payload_axis_tkeep            (ping_rx_ip_payload_axis_tkeep),
        .s_ip_payload_axis_tvalid           (ping_rx_ip_payload_axis_tvalid),
        .s_ip_payload_axis_tready           (ping_rx_ip_payload_axis_tready),
        .s_ip_payload_axis_tlast            (ping_rx_ip_payload_axis_tlast),
        .s_ip_payload_axis_tuser            (ping_rx_ip_payload_axis_tuser),

        .m_ip_hdr_valid                     (ping_tx_ip_hdr_valid),
        .m_ip_hdr_ready                     (ping_tx_ip_hdr_ready),
        .m_eth_dest_mac                     (ping_tx_eth_dest_mac),
        .m_eth_src_mac                      (ping_tx_eth_src_mac),
        .m_eth_type                         (ping_tx_eth_type),
        .m_ip_version                       (ping_tx_ip_version),
        .m_ip_ihl                           (ping_tx_ip_ihl),
        .m_ip_dscp                          (ping_tx_ip_dscp),
        .m_ip_ecn                           (ping_tx_ip_ecn),
        .m_ip_length                        (ping_tx_ip_length),
        .m_ip_identification                (ping_tx_ip_identification),
        .m_ip_flags                         (ping_tx_ip_flags),
        .m_ip_fragment_offset               (ping_tx_ip_fragment_offset),
        .m_ip_ttl                           (ping_tx_ip_ttl),
        .m_ip_protocol                      (ping_tx_ip_protocol),
        .m_ip_header_checksum               (ping_tx_ip_header_checksum),
        .m_ip_source_ip                     (ping_tx_ip_source_ip),
        .m_ip_dest_ip                       (ping_tx_ip_dest_ip),
        .m_ip_payload_axis_tdata            (ping_tx_ip_payload_axis_tdata),
        .m_ip_payload_axis_tkeep            (ping_tx_ip_payload_axis_tkeep),
        .m_ip_payload_axis_tvalid           (ping_tx_ip_payload_axis_tvalid),
        .m_ip_payload_axis_tready           (ping_tx_ip_payload_axis_tready),
        .m_ip_payload_axis_tlast            (ping_tx_ip_payload_axis_tlast),
        .m_ip_payload_axis_tuser            (ping_tx_ip_payload_axis_tuser)
    );

    ip_arb_mux #(
        .S_COUNT(2),
        .DATA_WIDTH(DATA_WIDTH)
    ) u_l3_tx_ip_arb_mux (
        .clk                                (clk),
        .rst                                (rst),

        .s_ip_hdr_valid                     ({ctrl_tx_ip_hdr_valid,             ping_tx_ip_hdr_valid            }),
        .s_ip_hdr_ready                     ({ctrl_tx_ip_hdr_ready,             ping_tx_ip_hdr_ready            }),
        .s_eth_dest_mac                     ({ctrl_tx_eth_dest_mac,             ping_tx_eth_dest_mac            }),
        .s_eth_src_mac                      ({ctrl_tx_eth_src_mac,              ping_tx_eth_src_mac             }),
        .s_eth_type                         ({ctrl_tx_eth_type,                 ping_tx_eth_type                }),
        .s_ip_version                       ({ctrl_tx_ip_version,               ping_tx_ip_version              }),
        .s_ip_ihl                           ({ctrl_tx_ip_ihl,                   ping_tx_ip_ihl                  }),
        .s_ip_dscp                          ({ctrl_tx_ip_dscp,                  ping_tx_ip_dscp                 }),
        .s_ip_ecn                           ({ctrl_tx_ip_ecn,                   ping_tx_ip_ecn                  }),
        .s_ip_length                        ({ctrl_tx_ip_length,                ping_tx_ip_length               }),
        .s_ip_identification                ({ctrl_tx_ip_identification,        ping_tx_ip_identification       }),
        .s_ip_flags                         ({ctrl_tx_ip_flags,                 ping_tx_ip_flags                }),
        .s_ip_fragment_offset               ({ctrl_tx_ip_fragment_offset,       ping_tx_ip_fragment_offset      }),
        .s_ip_ttl                           ({ctrl_tx_ip_ttl,                   ping_tx_ip_ttl                  }),
        .s_ip_protocol                      ({ctrl_tx_ip_protocol,              ping_tx_ip_protocol             }),
        .s_ip_header_checksum               ({ctrl_tx_ip_header_checksum,       ping_tx_ip_header_checksum      }),
        .s_ip_source_ip                     ({ctrl_tx_ip_source_ip,             ping_tx_ip_source_ip            }),
        .s_ip_dest_ip                       ({ctrl_tx_ip_dest_ip,               ping_tx_ip_dest_ip              }),
        .s_ip_payload_axis_tdata            ({ctrl_tx_ip_payload_axis_tdata,    ping_tx_ip_payload_axis_tdata   }),
        .s_ip_payload_axis_tkeep            ({ctrl_tx_ip_payload_axis_tkeep,    ping_tx_ip_payload_axis_tkeep   }),
        .s_ip_payload_axis_tvalid           ({ctrl_tx_ip_payload_axis_tvalid,   ping_tx_ip_payload_axis_tvalid  }),
        .s_ip_payload_axis_tready           ({ctrl_tx_ip_payload_axis_tready,   ping_tx_ip_payload_axis_tready  }),
        .s_ip_payload_axis_tlast            ({ctrl_tx_ip_payload_axis_tlast,    ping_tx_ip_payload_axis_tlast   }),
        .s_ip_payload_axis_tid              ('0),
        .s_ip_payload_axis_tdest            ('0),
        .s_ip_payload_axis_tuser            ({ctrl_tx_ip_payload_axis_tuser,    ping_tx_ip_payload_axis_tuser   }),

        .m_ip_hdr_valid                     (tx_ip_hdr_valid),
        .m_ip_hdr_ready                     (tx_ip_hdr_ready),
        .m_eth_dest_mac                     (tx_eth_dest_mac),
        .m_eth_src_mac                      (tx_eth_src_mac),
        .m_eth_type                         (tx_eth_type),
        .m_ip_version                       (tx_ip_version),
        .m_ip_ihl                           (tx_ip_ihl),
        .m_ip_dscp                          (tx_ip_dscp),
        .m_ip_ecn                           (tx_ip_ecn),
        .m_ip_length                        (tx_ip_length),
        .m_ip_identification                (tx_ip_identification),
        .m_ip_flags                         (tx_ip_flags),
        .m_ip_fragment_offset               (tx_ip_fragment_offset),
        .m_ip_ttl                           (tx_ip_ttl),
        .m_ip_protocol                      (tx_ip_protocol),
        .m_ip_header_checksum               (tx_ip_header_checksum),
        .m_ip_source_ip                     (tx_ip_source_ip),
        .m_ip_dest_ip                       (tx_ip_dest_ip),
        .m_ip_payload_axis_tdata            (tx_ip_payload_axis_tdata),
        .m_ip_payload_axis_tkeep            (tx_ip_payload_axis_tkeep),
        .m_ip_payload_axis_tvalid           (tx_ip_payload_axis_tvalid),
        .m_ip_payload_axis_tready           (tx_ip_payload_axis_tready),
        .m_ip_payload_axis_tlast            (tx_ip_payload_axis_tlast),
        .m_ip_payload_axis_tid              (),
        .m_ip_payload_axis_tdest            (),
        .m_ip_payload_axis_tuser            (tx_ip_payload_axis_tuser)
    );

generate
    if (DATA_WIDTH == 64) begin
        ip_eth_tx_64 u_l3_ip_eth_tx_64 (
            .clk                                (clk),
            .rst                                (rst),

            .s_ip_hdr_valid                     (tx_ip_hdr_valid),
            .s_ip_hdr_ready                     (tx_ip_hdr_ready),
            .s_eth_dest_mac                     (tx_eth_dest_mac),
            .s_eth_src_mac                      (tx_eth_src_mac),
            .s_eth_type                         (tx_eth_type),
            // .s_ip_version                       (tx_ip_version),
            // .s_ip_ihl                           (tx_ip_ihl),
            .s_ip_dscp                          (tx_ip_dscp),
            .s_ip_ecn                           (tx_ip_ecn),
            .s_ip_length                        (tx_ip_length),
            .s_ip_identification                (tx_ip_identification),
            .s_ip_flags                         (tx_ip_flags),
            .s_ip_fragment_offset               (tx_ip_fragment_offset),
            .s_ip_ttl                           (tx_ip_ttl),
            .s_ip_protocol                      (tx_ip_protocol),
            // .s_ip_header_checksum               (tx_ip_header_checksum),
            .s_ip_source_ip                     (tx_ip_source_ip),
            .s_ip_dest_ip                       (tx_ip_dest_ip),
            .s_ip_payload_axis_tdata            (tx_ip_payload_axis_tdata),
            .s_ip_payload_axis_tkeep            (tx_ip_payload_axis_tkeep),
            .s_ip_payload_axis_tvalid           (tx_ip_payload_axis_tvalid),
            .s_ip_payload_axis_tready           (tx_ip_payload_axis_tready),
            .s_ip_payload_axis_tlast            (tx_ip_payload_axis_tlast),
            .s_ip_payload_axis_tuser            (tx_ip_payload_axis_tuser),

            .m_eth_hdr_valid                    (ip_tx_eth_hdr_valid),
            .m_eth_hdr_ready                    (ip_tx_eth_hdr_ready),
            .m_eth_dest_mac                     (ip_tx_eth_dest_mac),
            .m_eth_src_mac                      (ip_tx_eth_src_mac),
            .m_eth_type                         (ip_tx_eth_type),
            .m_eth_payload_axis_tdata           (ip_tx_eth_payload_axis_tdata),
            .m_eth_payload_axis_tkeep           (ip_tx_eth_payload_axis_tkeep),
            .m_eth_payload_axis_tvalid          (ip_tx_eth_payload_axis_tvalid),
            .m_eth_payload_axis_tready          (ip_tx_eth_payload_axis_tready),
            .m_eth_payload_axis_tlast           (ip_tx_eth_payload_axis_tlast),
            .m_eth_payload_axis_tuser           (ip_tx_eth_payload_axis_tuser),

            .busy                               (tx_busy),
            .error_payload_early_termination    (tx_error_payload_early_termination)
        );
    end else if (DATA_WIDTH == 8) begin
       ip_eth_tx u_l3_ip_eth_tx (
            .clk                                (clk),
            .rst                                (rst),

            .s_ip_hdr_valid                     (tx_ip_hdr_valid),
            .s_ip_hdr_ready                     (tx_ip_hdr_ready),
            .s_eth_dest_mac                     (tx_eth_dest_mac),
            .s_eth_src_mac                      (tx_eth_src_mac),
            .s_eth_type                         (tx_eth_type),
            // .s_ip_version                       (tx_ip_version),
            // .s_ip_ihl                           (tx_ip_ihl),
            .s_ip_dscp                          (tx_ip_dscp),
            .s_ip_ecn                           (tx_ip_ecn),
            .s_ip_length                        (tx_ip_length),
            .s_ip_identification                (tx_ip_identification),
            .s_ip_flags                         (tx_ip_flags),
            .s_ip_fragment_offset               (tx_ip_fragment_offset),
            .s_ip_ttl                           (tx_ip_ttl),
            .s_ip_protocol                      (tx_ip_protocol),
            // .s_ip_header_checksum               (tx_ip_header_checksum),
            .s_ip_source_ip                     (tx_ip_source_ip),
            .s_ip_dest_ip                       (tx_ip_dest_ip),
            .s_ip_payload_axis_tdata            (tx_ip_payload_axis_tdata),
            .s_ip_payload_axis_tvalid           (tx_ip_payload_axis_tvalid),
            .s_ip_payload_axis_tready           (tx_ip_payload_axis_tready),
            .s_ip_payload_axis_tlast            (tx_ip_payload_axis_tlast),
            .s_ip_payload_axis_tuser            (tx_ip_payload_axis_tuser),

            .m_eth_hdr_valid                    (ip_tx_eth_hdr_valid),
            .m_eth_hdr_ready                    (ip_tx_eth_hdr_ready),
            .m_eth_dest_mac                     (ip_tx_eth_dest_mac),
            .m_eth_src_mac                      (ip_tx_eth_src_mac),
            .m_eth_type                         (ip_tx_eth_type),
            .m_eth_payload_axis_tdata           (ip_tx_eth_payload_axis_tdata),
            .m_eth_payload_axis_tvalid          (ip_tx_eth_payload_axis_tvalid),
            .m_eth_payload_axis_tready          (ip_tx_eth_payload_axis_tready),
            .m_eth_payload_axis_tlast           (ip_tx_eth_payload_axis_tlast),
            .m_eth_payload_axis_tuser           (ip_tx_eth_payload_axis_tuser),

            .busy                               (tx_busy),
            .error_payload_early_termination    (tx_error_payload_early_termination)
        );

        assign ip_tx_eth_payload_axis_tkeep = m_eth_payload_axis_tvalid;
    end else begin
        $error("Only 8 and 64 bit data widths are supported");
    end
endgenerate

    eth_arb_mux #(
        .S_COUNT        (2),
        .DATA_WIDTH     (DATA_WIDTH),
        .ID_ENABLE      (0),
        .DEST_ENABLE    (0)
    ) u_ntw_eth_tx_mux (
        .clk                                (clk),
        .rst                                (rst),

        .s_eth_hdr_valid                    ({arp_tx_eth_hdr_valid,              ip_tx_eth_hdr_valid             }),
        .s_eth_hdr_ready                    ({arp_tx_eth_hdr_ready,              ip_tx_eth_hdr_ready             }),
        .s_eth_dest_mac                     ({arp_tx_eth_dest_mac,               ip_tx_eth_dest_mac              }),
        .s_eth_src_mac                      ({arp_tx_eth_src_mac,                ip_tx_eth_src_mac               }),
        .s_eth_type                         ({arp_tx_eth_type,                   ip_tx_eth_type                  }),
        .s_eth_payload_axis_tdata           ({arp_tx_eth_payload_axis_tdata,     ip_tx_eth_payload_axis_tdata    }),
        .s_eth_payload_axis_tkeep           ({arp_tx_eth_payload_axis_tkeep,     ip_tx_eth_payload_axis_tkeep    }),
        .s_eth_payload_axis_tvalid          ({arp_tx_eth_payload_axis_tvalid,    ip_tx_eth_payload_axis_tvalid   }),
        .s_eth_payload_axis_tready          ({arp_tx_eth_payload_axis_tready,    ip_tx_eth_payload_axis_tready   }),
        .s_eth_payload_axis_tlast           ({arp_tx_eth_payload_axis_tlast,     ip_tx_eth_payload_axis_tlast    }),
        .s_eth_payload_axis_tid             ('0),
        .s_eth_payload_axis_tdest           ('0),
        .s_eth_payload_axis_tuser           ({arp_tx_eth_payload_axis_tuser,     ip_tx_eth_payload_axis_tuser    }),

        .m_eth_hdr_valid                    (m_eth_hdr_valid),
        .m_eth_hdr_ready                    (m_eth_hdr_ready),
        .m_eth_dest_mac                     (m_eth_dest_mac),
        .m_eth_src_mac                      (m_eth_src_mac),
        .m_eth_type                         (m_eth_type),
        .m_eth_payload_axis_tdata           (m_eth_payload_axis_tdata),
        .m_eth_payload_axis_tkeep           (m_eth_payload_axis_tkeep),
        .m_eth_payload_axis_tvalid          (m_eth_payload_axis_tvalid),
        .m_eth_payload_axis_tready          (m_eth_payload_axis_tready),
        .m_eth_payload_axis_tlast           (m_eth_payload_axis_tlast),
        .m_eth_payload_axis_tid             (),
        .m_eth_payload_axis_tdest           (),
        .m_eth_payload_axis_tuser           (m_eth_payload_axis_tuser)
    );

endmodule
