module ping_client #(
    parameter DATA_WIDTH = 8,
    parameter KEEP_WIDTH = (DATA_WIDTH/8),
    parameter LOCAL_MAC = 48'h020000aabbcc, // should be a register setting
    parameter LOCAL_IP  = 32'hac000002      // should be a register setting
)
(
    input  wire        clk,
    input  wire        rst,

    /*
     * ICMP frame input
     */
    input  wire                     s_icmp_hdr_valid,
    output wire                     s_icmp_hdr_ready,
    input  wire [47:0]              s_eth_dest_mac,
    input  wire [47:0]              s_eth_src_mac,
    input  wire [15:0]              s_eth_type,
    input  wire [3:0]               s_ip_version,
    input  wire [3:0]               s_ip_ihl,
    input  wire [5:0]               s_ip_dscp,
    input  wire [1:0]               s_ip_ecn,
    input  wire [15:0]              s_ip_length,
    input  wire [15:0]              s_ip_identification,
    input  wire [2:0]               s_ip_flags,
    input  wire [12:0]              s_ip_fragment_offset,
    input  wire [7:0]               s_ip_ttl,
    input  wire [7:0]               s_ip_protocol,
    input  wire [15:0]              s_ip_header_checksum,
    input  wire [31:0]              s_ip_source_ip,
    input  wire [31:0]              s_ip_dest_ip,
    input  wire [7:0]               s_icmp_type,
    input  wire [7:0]               s_icmp_code,
    input  wire [15:0]              s_icmp_checksum,
    input  wire [31:0]              s_icmp_rest_of_header,
    input  wire [DATA_WIDTH-1:0]    s_icmp_payload_axis_tdata,
    input  wire [KEEP_WIDTH-1:0]    s_icmp_payload_axis_tkeep,
    input  wire                     s_icmp_payload_axis_tvalid,
    output wire                     s_icmp_payload_axis_tready,
    input  wire                     s_icmp_payload_axis_tlast,
    input  wire                     s_icmp_payload_axis_tuser,

    /*
     * ICMP frame output
     */
    output wire                     m_icmp_hdr_valid,
    input  wire                     m_icmp_hdr_ready,
    output wire [47:0]              m_eth_dest_mac,
    output wire [47:0]              m_eth_src_mac,
    output wire [15:0]              m_eth_type,
    output wire [3:0]               m_ip_version,
    output wire [3:0]               m_ip_ihl,
    output wire [5:0]               m_ip_dscp,
    output wire [1:0]               m_ip_ecn,
    output wire [15:0]              m_ip_length,
    output wire [15:0]              m_ip_identification,
    output wire [2:0]               m_ip_flags,
    output wire [12:0]              m_ip_fragment_offset,
    output wire [7:0]               m_ip_ttl,
    output wire [7:0]               m_ip_protocol,
    output wire [15:0]              m_ip_header_checksum,
    output wire [31:0]              m_ip_source_ip,
    output wire [31:0]              m_ip_dest_ip,
    output wire [7:0]               m_icmp_type,
    output wire [7:0]               m_icmp_code,
    output wire [15:0]              m_icmp_checksum,
    output wire [31:0]              m_icmp_rest_of_header,
    output wire [DATA_WIDTH-1:0]    m_icmp_payload_axis_tdata,
    output wire [KEEP_WIDTH-1:0]    m_icmp_payload_axis_tkeep,
    output wire                     m_icmp_payload_axis_tvalid,
    input  wire                     m_icmp_payload_axis_tready,
    output wire                     m_icmp_payload_axis_tlast,
    output wire                     m_icmp_payload_axis_tuser
);

    localparam HDR_FIFO_WIDTH = (16+16+16+32+48+16+16+16);
    localparam DATA_FIFO_WIDTH = (DATA_WIDTH+KEEP_WIDTH+1+1);

    enum logic {HEADER, DATA} tx_state, tx_state_next;

    logic [15:0] ip_new_checksum;
    logic [15:0] icmp_new_checksum;


    logic hdr_fifo_full;
    assign s_icmp_hdr_ready = ~hdr_fifo_full;

    logic data_fifo_full;
    assign s_icmp_payload_axis_tready = ~data_fifo_full;

    logic hdr_fifo_empty;
    logic data_fifo_empty;


    // for IP header the only thing we really need to change is the TTL.
    // Since the source and dest ip addresses are flipped, the sum stays the same.
    checksum_mod u_ping_client_ip_checksum_mod (
        .i_checksum (s_ip_header_checksum),
        .i_old_data ({s_ip_ttl, s_ip_protocol}),
        .i_new_data ({8'h40, s_ip_protocol}),
        .o_checksum (ip_new_checksum)
    );

    checksum_mod u_ping_client_icmp_checksum_mod (
        .i_checksum (s_icmp_checksum),
        .i_old_data ({s_icmp_type, s_icmp_code}),
        .i_new_data ({8'h0, s_icmp_code}),
        .o_checksum (icmp_new_checksum)
    );

    // Header fifo stores:
    // IP Total Length      16
    // IP Identification    16
    // IP Checksum          16 (modified)
    // IP Dest Address      32 (originally the source address)
    // ETH Dest Address     48 (originally the source address)
    // ICMP Sequence        16 (see below
    // ICMP Identifier      16 (This and above are stored as rest_of_header)
    // ICMP Checksum        16 (modified)


    fifo_fwft #(
        .DEPTH_WIDTH(2),
        .DATA_WIDTH(HDR_FIFO_WIDTH)
    ) u_ping_client_header_fifo (
        .rst        (rst),
        .clk        (clk),

        .wr_en    (s_icmp_hdr_valid),
        .din  ({
                        s_ip_length,
                        s_ip_identification,
                        ip_new_checksum,
                        s_ip_source_ip,
                        s_eth_src_mac,
                        s_icmp_rest_of_header,
                        icmp_new_checksum
                    }),
        .full     (hdr_fifo_full),

        .rd_en    (m_icmp_hdr_valid),
        .dout  ({
                        m_ip_length,
                        m_ip_identification,
                        m_ip_header_checksum,
                        m_ip_dest_ip,
                        m_eth_dest_mac,
                        m_icmp_rest_of_header,
                        m_icmp_checksum
                    }),
        .empty    (hdr_fifo_empty)
    );

    fifo_fwft #(
        .DEPTH_WIDTH(7),
        .DATA_WIDTH(DATA_FIFO_WIDTH)
    ) u_ping_client_data_fifo (
        .rst        (rst),
        .clk        (clk),

        .wr_en      (s_icmp_payload_axis_tvalid),
        .din        ({
                        s_icmp_payload_axis_tdata,
                        s_icmp_payload_axis_tkeep,
                        s_icmp_payload_axis_tuser,
                        s_icmp_payload_axis_tlast
                    }),
        .full       (data_fifo_full),

        .rd_en      (icmp_payload_valid),
        .dout       ({
                        m_icmp_payload_axis_tdata,
                        m_icmp_payload_axis_tkeep,
                        m_icmp_payload_axis_tuser,
                        m_icmp_payload_axis_tlast
                    }),
        .empty      (data_fifo_empty)
    );


    logic icmp_hdr_valid;
    logic icmp_payload_valid;

    assign m_icmp_hdr_valid = icmp_hdr_valid;
    assign m_icmp_payload_axis_tvalid = icmp_payload_valid;

    assign m_eth_src_mac = LOCAL_MAC;
    assign m_eth_type = 16'h0800;
    assign m_icmp_code = 8'h0;
    assign m_icmp_type = 8'h0;
    assign m_ip_dscp = 6'h0;
    assign m_ip_ecn = 2'h0;
    assign m_ip_flags = 3'b010;
    assign m_ip_fragment_offset = 13'h0;
    assign m_ip_ttl = 8'h40;
    assign m_ip_protocol = 8'h1;
    assign m_ip_version = 4'h4;
    assign m_ip_ihl = 4'h5;
    assign m_ip_source_ip = LOCAL_IP;

    always_ff @(posedge clk) begin
        if (rst) begin
            tx_state <= HEADER;
        end else begin
            tx_state <= tx_state_next;
        end
    end

    always_comb begin
        tx_state_next = tx_state;

        icmp_hdr_valid = '0;
        icmp_payload_valid = '0;

        case (tx_state)
            HEADER: begin
                icmp_hdr_valid = m_icmp_hdr_ready & ~hdr_fifo_empty;

                if (icmp_hdr_valid) begin
                    tx_state_next = DATA;
                end
            end

            DATA: begin
                icmp_payload_valid = m_icmp_payload_axis_tready & ~data_fifo_empty;
                if (icmp_payload_valid & m_icmp_payload_axis_tlast) begin
                    tx_state_next = HEADER;
                end
            end
        endcase
    end

endmodule