module network_top #(
    parameter AXIS_DATA_WIDTH = 8,
    parameter AXIS_KEEP_WIDTH = AXIS_DATA_WIDTH/8
)
(
    input                           clk,
    input                           rst,

    input                           eth_rx_axis_tvalid,
    output                          eth_rx_axis_tready,
    input   [AXIS_DATA_WIDTH-1:0]   eth_rx_axis_tdata,
    input   [AXIS_KEEP_WIDTH-1:0]   eth_rx_axis_tkeep,
    input                           eth_rx_axis_tlast,

    output                          eth_tx_axis_tvalid,
    input                           eth_tx_axis_tready,
    output  [AXIS_DATA_WIDTH-1:0]   eth_tx_axis_tdata,
    output  [AXIS_KEEP_WIDTH-1:0]   eth_tx_axis_tkeep,
    output                          eth_tx_axis_tlast,

    input                           modem_rx_axis_tvalid,
    output                          modem_rx_axis_tready,
    input   [AXIS_DATA_WIDTH-1:0]   modem_rx_axis_tdata,
    input   [AXIS_KEEP_WIDTH-1:0]   modem_rx_axis_tkeep,
    input                           modem_rx_axis_tlast,

    output                          modem_tx_axis_tvalid,
    input                           modem_tx_axis_tready,
    output  [AXIS_DATA_WIDTH-1:0]   modem_tx_axis_tdata,
    output  [AXIS_KEEP_WIDTH-1:0]   modem_tx_axis_tkeep,
    output                          modem_tx_axis_tlast,

    // APB Master
    output wire [31:0]                  m_apb_paddr,
    output wire                         m_apb_psel,
    output wire                         m_apb_penable,
    output wire                         m_apb_pwrite,
    output wire [31:0]                  m_apb_pwdata,
    output wire [3:0]                   m_apb_pstrb,
    input  wire                         m_apb_pready,
    input  wire [31:0]                  m_apb_prdata,
    input  wire                         m_apb_pslverr
);


    localparam LOCAL_MAC = 48'h020000aabbcc;

    logic                           eth_rx_eth_hdr_valid;
    logic                           eth_rx_eth_hdr_ready;
    logic   [47:0]                  eth_rx_eth_dest_mac;
    logic   [47:0]                  eth_rx_eth_src_mac;
    logic   [15:0]                  eth_rx_eth_type;
    logic   [AXIS_DATA_WIDTH-1:0]   eth_rx_eth_payload_axis_tdata;
    logic   [AXIS_KEEP_WIDTH-1:0]   eth_rx_eth_payload_axis_tkeep;
    logic                           eth_rx_eth_payload_axis_tvalid;
    logic                           eth_rx_eth_payload_axis_tready;
    logic                           eth_rx_eth_payload_axis_tlast;
    logic                           eth_rx_eth_payload_axis_tuser;

    logic                           eth_rx_eth_busy;
    logic                           eth_rx_eth_error;

    logic                           eth_mux_eth_hdr_valid;
    logic                           eth_mux_eth_hdr_ready;
    logic   [47:0]                  eth_mux_eth_dest_mac;
    logic   [47:0]                  eth_mux_eth_src_mac;
    logic   [15:0]                  eth_mux_eth_type;
    logic   [AXIS_DATA_WIDTH-1:0]   eth_mux_eth_payload_axis_tdata;
    logic   [AXIS_KEEP_WIDTH-1:0]   eth_mux_eth_payload_axis_tkeep;
    logic                           eth_mux_eth_payload_axis_tvalid;
    logic                           eth_mux_eth_payload_axis_tready;
    logic                           eth_mux_eth_payload_axis_tlast;
    logic                           eth_mux_eth_payload_axis_tuser;


    logic                           fwd_eth_hdr_valid;
    logic                           fwd_eth_hdr_ready;
    logic   [47:0]                  fwd_eth_dest_mac;
    logic   [47:0]                  fwd_eth_src_mac;
    logic   [15:0]                  fwd_eth_type;
    logic   [AXIS_DATA_WIDTH-1:0]   fwd_eth_payload_axis_tdata;
    logic   [AXIS_KEEP_WIDTH-1:0]   fwd_eth_payload_axis_tkeep;
    logic                           fwd_eth_payload_axis_tvalid;
    logic                           fwd_eth_payload_axis_tready;
    logic                           fwd_eth_payload_axis_tlast;
    logic                           fwd_eth_payload_axis_tuser;

    logic                           local_eth_hdr_valid;
    logic                           local_eth_hdr_ready;
    logic   [47:0]                  local_eth_dest_mac;
    logic   [47:0]                  local_eth_src_mac;
    logic   [15:0]                  local_eth_type;
    logic   [AXIS_DATA_WIDTH-1:0]   local_eth_payload_axis_tdata;
    logic   [AXIS_KEEP_WIDTH-1:0]   local_eth_payload_axis_tkeep;
    logic                           local_eth_payload_axis_tvalid;
    logic                           local_eth_payload_axis_tready;
    logic                           local_eth_payload_axis_tlast;
    logic                           local_eth_payload_axis_tuser;

    logic                           eth_rx_demux_sel;

    logic                           l3_tx_eth_hdr_valid;
    logic                           l3_tx_eth_hdr_ready;
    logic   [47:0]                  l3_tx_eth_dest_mac;
    logic   [47:0]                  l3_tx_eth_src_mac;
    logic   [15:0]                  l3_tx_eth_type;
    logic   [AXIS_DATA_WIDTH-1:0]   l3_tx_eth_payload_axis_tdata;
    logic   [AXIS_KEEP_WIDTH-1:0]   l3_tx_eth_payload_axis_tkeep;
    logic                           l3_tx_eth_payload_axis_tvalid;
    logic                           l3_tx_eth_payload_axis_tready;
    logic                           l3_tx_eth_payload_axis_tlast;
    logic                           l3_tx_eth_payload_axis_tuser;

    logic                           modem_rx_eth_hdr_valid;
    logic                           modem_rx_eth_hdr_ready;
    logic   [47:0]                  modem_rx_eth_dest_mac;
    logic   [47:0]                  modem_rx_eth_src_mac;
    logic   [15:0]                  modem_rx_eth_type;
    logic   [AXIS_DATA_WIDTH-1:0]   modem_rx_eth_payload_axis_tdata;
    logic   [AXIS_KEEP_WIDTH-1:0]   modem_rx_eth_payload_axis_tkeep;
    logic                           modem_rx_eth_payload_axis_tvalid;
    logic                           modem_rx_eth_payload_axis_tready;
    logic                           modem_rx_eth_payload_axis_tlast;
    logic                           modem_rx_eth_payload_axis_tuser;

    logic                           eth_tx_eth_hdr_valid;
    logic                           eth_tx_eth_hdr_ready;
    logic   [47:0]                  eth_tx_eth_dest_mac;
    logic   [47:0]                  eth_tx_eth_src_mac;
    logic   [15:0]                  eth_tx_eth_type;
    logic   [AXIS_DATA_WIDTH-1:0]   eth_tx_eth_payload_axis_tdata;
    logic   [AXIS_KEEP_WIDTH-1:0]   eth_tx_eth_payload_axis_tkeep;
    logic                           eth_tx_eth_payload_axis_tvalid;
    logic                           eth_tx_eth_payload_axis_tready;
    logic                           eth_tx_eth_payload_axis_tlast;
    logic                           eth_tx_eth_payload_axis_tuser;

    logic                           eth_tx_busy;


    eth_axis_rx #(
        .DATA_WIDTH     (AXIS_DATA_WIDTH)
    ) u_ntw_eth_axis_rx_incoming (
        .clk                                (clk),
        .rst                                (rst),

        .s_axis_tdata                       (eth_rx_axis_tdata),
        .s_axis_tkeep                       (eth_rx_axis_tkeep),
        .s_axis_tvalid                      (eth_rx_axis_tvalid),
        .s_axis_tready                      (eth_rx_axis_tready),
        .s_axis_tlast                       (eth_rx_axis_tlast),
        .s_axis_tuser                       ('0),

        .m_eth_hdr_valid                    (eth_rx_eth_hdr_valid),
        .m_eth_hdr_ready                    (eth_rx_eth_hdr_ready),
        .m_eth_dest_mac                     (eth_rx_eth_dest_mac),
        .m_eth_src_mac                      (eth_rx_eth_src_mac),
        .m_eth_type                         (eth_rx_eth_type),
        .m_eth_payload_axis_tdata           (eth_rx_eth_payload_axis_tdata),
        .m_eth_payload_axis_tkeep           (eth_rx_eth_payload_axis_tkeep),
        .m_eth_payload_axis_tvalid          (eth_rx_eth_payload_axis_tvalid),
        .m_eth_payload_axis_tready          (eth_rx_eth_payload_axis_tready),
        .m_eth_payload_axis_tlast           (eth_rx_eth_payload_axis_tlast),
        .m_eth_payload_axis_tuser           (eth_rx_eth_payload_axis_tuser),

        .busy                               (eth_rx_eth_busy),
        .error_header_early_termination     (eth_rx_eth_error)
    );

    eth_duplicator #(
        .DATA_WIDTH     (AXIS_DATA_WIDTH),
        .ID_ENABLE      (0),
        .DEST_ENABLE    (0)
    ) u_eth_duplicator (
        .clk                                (clk),
        .rst                                (rst),

        .s_eth_hdr_valid                    (eth_rx_eth_hdr_valid),
        .s_eth_hdr_ready                    (eth_rx_eth_hdr_ready),
        .s_eth_dest_mac                     (eth_rx_eth_dest_mac),
        .s_eth_src_mac                      (eth_rx_eth_src_mac),
        .s_eth_type                         (eth_rx_eth_type),
        .s_eth_payload_axis_tdata           (eth_rx_eth_payload_axis_tdata),
        .s_eth_payload_axis_tkeep           (eth_rx_eth_payload_axis_tkeep),
        .s_eth_payload_axis_tvalid          (eth_rx_eth_payload_axis_tvalid),
        .s_eth_payload_axis_tready          (eth_rx_eth_payload_axis_tready),
        .s_eth_payload_axis_tlast           (eth_rx_eth_payload_axis_tlast),
        .s_eth_payload_axis_tuser           (eth_rx_eth_payload_axis_tuser),

        .m_eth_hdr_valid                    (eth_mux_eth_hdr_valid),
        .m_eth_hdr_ready                    (eth_mux_eth_hdr_ready),
        .m_eth_dest_mac                     (eth_mux_eth_dest_mac),
        .m_eth_src_mac                      (eth_mux_eth_src_mac),
        .m_eth_type                         (eth_mux_eth_type),
        .m_eth_payload_axis_tdata           (eth_mux_eth_payload_axis_tdata),
        .m_eth_payload_axis_tkeep           (eth_mux_eth_payload_axis_tkeep),
        .m_eth_payload_axis_tvalid          (eth_mux_eth_payload_axis_tvalid),
        .m_eth_payload_axis_tready          (eth_mux_eth_payload_axis_tready),
        .m_eth_payload_axis_tlast           (eth_mux_eth_payload_axis_tlast),
        .m_eth_payload_axis_tuser           (eth_mux_eth_payload_axis_tuser),


        // These could all be register settings.
        .source_mac_filter                  ('0),
        .source_mac_filter_mask             ('0),
        .source_mac_filter_en               ('0),
        .dest_mac_filter                    (48'hffffffffffff),
        .dest_mac_filter_mask               (48'hffffffffffff),
        .dest_mac_filter_en                 ('1),
        .ethertype_filter                   ('0),
        .ethertype_filter_mask              ('0),
        .ethertype_filter_en                ('0),

        .source_mac_mod                     ('0),
        .source_mac_mod_mask                ('0),
        .source_mac_mod_en                  ('0),
        .dest_mac_mod                       (LOCAL_MAC),
        .dest_mac_mod_mask                  (48'hffffffffffff),
        .dest_mac_mod_en                    ('1),
        .ethertype_mod                      ('0),
        .ethertype_mod_mask                 ('0),
        .ethertype_mod_en                   ('0)
    );

    assign eth_rx_demux_sel = (eth_mux_eth_dest_mac == LOCAL_MAC) ? 0 : 1;

    eth_demux #(
        .M_COUNT        (2),
        .DATA_WIDTH     (AXIS_DATA_WIDTH),
        .ID_ENABLE      (0),
        .DEST_ENABLE    (0)
    ) u_ntw_eth_rx_demux (
        .clk                                (clk),
        .rst                                (rst),

        .s_eth_hdr_valid                    (eth_mux_eth_hdr_valid),
        .s_eth_hdr_ready                    (eth_mux_eth_hdr_ready),
        .s_eth_dest_mac                     (eth_mux_eth_dest_mac),
        .s_eth_src_mac                      (eth_mux_eth_src_mac),
        .s_eth_type                         (eth_mux_eth_type),
        .s_eth_payload_axis_tdata           (eth_mux_eth_payload_axis_tdata),
        .s_eth_payload_axis_tkeep           (eth_mux_eth_payload_axis_tkeep),
        .s_eth_payload_axis_tvalid          (eth_mux_eth_payload_axis_tvalid),
        .s_eth_payload_axis_tready          (eth_mux_eth_payload_axis_tready),
        .s_eth_payload_axis_tlast           (eth_mux_eth_payload_axis_tlast),
        .s_eth_payload_axis_tid             ('0),
        .s_eth_payload_axis_tdest           ('0),
        .s_eth_payload_axis_tuser           (eth_mux_eth_payload_axis_tuser),

        .m_eth_hdr_valid                    ({fwd_eth_hdr_valid,            local_eth_hdr_valid             }),
        .m_eth_hdr_ready                    ({fwd_eth_hdr_ready,            local_eth_hdr_ready             }),
        .m_eth_dest_mac                     ({fwd_eth_dest_mac,             local_eth_dest_mac              }),
        .m_eth_src_mac                      ({fwd_eth_src_mac,              local_eth_src_mac               }),
        .m_eth_type                         ({fwd_eth_type,                 local_eth_type                  }),
        .m_eth_payload_axis_tdata           ({fwd_eth_payload_axis_tdata,   local_eth_payload_axis_tdata    }),
        .m_eth_payload_axis_tkeep           ({fwd_eth_payload_axis_tkeep,   local_eth_payload_axis_tkeep    }),
        .m_eth_payload_axis_tvalid          ({fwd_eth_payload_axis_tvalid,  local_eth_payload_axis_tvalid   }),
        .m_eth_payload_axis_tready          ({fwd_eth_payload_axis_tready,  local_eth_payload_axis_tready   }),
        .m_eth_payload_axis_tlast           ({fwd_eth_payload_axis_tlast,   local_eth_payload_axis_tlast    }),
        .m_eth_payload_axis_tid             (),
        .m_eth_payload_axis_tdest           (),
        .m_eth_payload_axis_tuser           ({fwd_eth_payload_axis_tuser,   local_eth_payload_axis_tuser    }),

        .enable                             ('1),
        .drop                               (eth_rx_eth_error), //?
        .select                             (eth_rx_demux_sel)
    );

    l3 #(
        .DATA_WIDTH(AXIS_DATA_WIDTH)
    ) u_l3 (
        .clk                                (clk),
        .rst                                (rst),

        .s_eth_hdr_valid                    (local_eth_hdr_valid),
        .s_eth_hdr_ready                    (local_eth_hdr_ready),
        .s_eth_dest_mac                     (local_eth_dest_mac),
        .s_eth_src_mac                      (local_eth_src_mac),
        .s_eth_type                         (local_eth_type),
        .s_eth_payload_axis_tdata           (local_eth_payload_axis_tdata),
        .s_eth_payload_axis_tkeep           (local_eth_payload_axis_tkeep),
        .s_eth_payload_axis_tvalid          (local_eth_payload_axis_tvalid),
        .s_eth_payload_axis_tready          (local_eth_payload_axis_tready),
        .s_eth_payload_axis_tlast           (local_eth_payload_axis_tlast),
        .s_eth_payload_axis_tuser           (local_eth_payload_axis_tuser),

        .m_eth_hdr_valid                    (l3_tx_eth_hdr_valid),
        .m_eth_hdr_ready                    (l3_tx_eth_hdr_ready),
        .m_eth_dest_mac                     (l3_tx_eth_dest_mac),
        .m_eth_src_mac                      (l3_tx_eth_src_mac),
        .m_eth_type                         (l3_tx_eth_type),
        .m_eth_payload_axis_tdata           (l3_tx_eth_payload_axis_tdata),
        .m_eth_payload_axis_tkeep           (l3_tx_eth_payload_axis_tkeep),
        .m_eth_payload_axis_tvalid          (l3_tx_eth_payload_axis_tvalid),
        .m_eth_payload_axis_tready          (l3_tx_eth_payload_axis_tready),
        .m_eth_payload_axis_tlast           (l3_tx_eth_payload_axis_tlast),
        .m_eth_payload_axis_tuser           (l3_tx_eth_payload_axis_tuser),

        .m_apb_paddr                        (m_apb_paddr),
        .m_apb_psel                         (m_apb_psel),
        .m_apb_penable                      (m_apb_penable),
        .m_apb_pwrite                       (m_apb_pwrite),
        .m_apb_pwdata                       (m_apb_pwdata),
        .m_apb_pstrb                        (m_apb_pstrb),
        .m_apb_pready                       (m_apb_pready),
        .m_apb_prdata                       (m_apb_prdata),
        .m_apb_pslverr                      (m_apb_pslverr)
    );

    assign modem_rx_eth_payload_axis_tvalid = '0;
    assign modem_rx_eth_hdr_valid = '0;

    eth_arb_mux #(
        .S_COUNT        (2),
        .DATA_WIDTH     (AXIS_DATA_WIDTH),
        .ID_ENABLE      (0),
        .DEST_ENABLE    (0)
    ) u_ntw_eth_tx_mux (
        .clk                                (clk),
        .rst                                (rst),

        .s_eth_hdr_valid                    ({l3_tx_eth_hdr_valid,              modem_rx_eth_hdr_valid             }),
        .s_eth_hdr_ready                    ({l3_tx_eth_hdr_ready,              modem_rx_eth_hdr_ready             }),
        .s_eth_dest_mac                     ({l3_tx_eth_dest_mac,               modem_rx_eth_dest_mac              }),
        .s_eth_src_mac                      ({l3_tx_eth_src_mac,                modem_rx_eth_src_mac               }),
        .s_eth_type                         ({l3_tx_eth_type,                   modem_rx_eth_type                  }),
        .s_eth_payload_axis_tdata           ({l3_tx_eth_payload_axis_tdata,     modem_rx_eth_payload_axis_tdata    }),
        .s_eth_payload_axis_tkeep           ({l3_tx_eth_payload_axis_tkeep,     modem_rx_eth_payload_axis_tkeep    }),
        .s_eth_payload_axis_tvalid          ({l3_tx_eth_payload_axis_tvalid,    modem_rx_eth_payload_axis_tvalid   }),
        .s_eth_payload_axis_tready          ({l3_tx_eth_payload_axis_tready,    modem_rx_eth_payload_axis_tready   }),
        .s_eth_payload_axis_tlast           ({l3_tx_eth_payload_axis_tlast,     modem_rx_eth_payload_axis_tlast    }),
        .s_eth_payload_axis_tid             ('0),
        .s_eth_payload_axis_tdest           ('0),
        .s_eth_payload_axis_tuser           ({l3_tx_eth_payload_axis_tuser,     modem_rx_eth_payload_axis_tuser    }),

        .m_eth_hdr_valid                    (eth_tx_eth_hdr_valid),
        .m_eth_hdr_ready                    (eth_tx_eth_hdr_ready),
        .m_eth_dest_mac                     (eth_tx_eth_dest_mac),
        .m_eth_src_mac                      (eth_tx_eth_src_mac),
        .m_eth_type                         (eth_tx_eth_type),
        .m_eth_payload_axis_tdata           (eth_tx_eth_payload_axis_tdata),
        .m_eth_payload_axis_tkeep           (eth_tx_eth_payload_axis_tkeep),
        .m_eth_payload_axis_tvalid          (eth_tx_eth_payload_axis_tvalid),
        .m_eth_payload_axis_tready          (eth_tx_eth_payload_axis_tready),
        .m_eth_payload_axis_tlast           (eth_tx_eth_payload_axis_tlast),
        .m_eth_payload_axis_tid             (),
        .m_eth_payload_axis_tdest           (),
        .m_eth_payload_axis_tuser           (eth_tx_eth_payload_axis_tuser)
    );

    eth_axis_tx #(
        .DATA_WIDTH     (AXIS_DATA_WIDTH)
    ) u_ntw_eth_axis_tx_outgoing (
        .clk                                (clk),
        .rst                                (rst),

        .s_eth_hdr_valid                    (eth_tx_eth_hdr_valid),
        .s_eth_hdr_ready                    (eth_tx_eth_hdr_ready),
        .s_eth_dest_mac                     (eth_tx_eth_dest_mac),
        .s_eth_src_mac                      (eth_tx_eth_src_mac),
        .s_eth_type                         (eth_tx_eth_type),
        .s_eth_payload_axis_tdata           (eth_tx_eth_payload_axis_tdata),
        .s_eth_payload_axis_tkeep           (eth_tx_eth_payload_axis_tkeep),
        .s_eth_payload_axis_tvalid          (eth_tx_eth_payload_axis_tvalid),
        .s_eth_payload_axis_tready          (eth_tx_eth_payload_axis_tready),
        .s_eth_payload_axis_tlast           (eth_tx_eth_payload_axis_tlast),
        .s_eth_payload_axis_tuser           (eth_tx_eth_payload_axis_tuser),

        .m_axis_tdata                       (eth_tx_axis_tdata),
        .m_axis_tkeep                       (eth_tx_axis_tkeep),
        .m_axis_tvalid                      (eth_tx_axis_tvalid),
        .m_axis_tready                      (eth_tx_axis_tready),
        .m_axis_tlast                       (eth_tx_axis_tlast),
        .m_axis_tuser                       (),

        .busy                               (eth_tx_eth_busy)
    );


    assign fwd_eth_payload_axis_tready = '1;
    assign fwd_eth_hdr_ready = '1;

endmodule